# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.application import Application as App_base


class Application(object):
    @staticmethod
    def list(auth):
        try:
            data = App_base._list(auth)
            applications_info = []
            applications = data.get("applications", [])
            users = data.get("users", {})
            for application in applications:
                application["creater_name"] = users.get(application.get("creater", ""), {}).get("nickname", "") \
                                              or users.get(application.get("creater", ""), {}).get("username", "")
                application_info = dict(id=application.get("id", ""),
                                        name=application.get("name", ""),
                                        lifecycle=application.get("lifecycle", ""),
                                        status_summary=application.get("status_summary", ""),
                                        running_status=application.get("running_status", ""),
                                        cpu_limit=application.get("cpu_limit", ""),
                                        memory_limit=application.get("memory_limit", ""),
                                        service_num=application.get("service_num", ""),
                                        service_running_num=application.get("service_running_num", ""),
                                        task_num=application.get("task_num", ""),
                                        task_running_num=application.get("task_running_num", ""),
                                        source=application.get("source", ""),
                                        creater_name=application.get("creater_name", ""),
                                        create_time=application.get("create_time", "")
                                        )
                applications_info.append(application_info)
            print json.dumps(dict(applications=applications_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, appid):
        try:
            data = App_base._show(auth, appid)
            application = data
            application_info = dict(id=application.get("id", ""),
                                    name=application.get("name", ""),
                                    lifecycle=application.get("lifecycle", ""),
                                    status_summary=application.get("status_summary", ""),
                                    running_status=application.get("running_status", ""),
                                    cpu_limit=application.get("cpu_limit", ""),
                                    memory_limit=application.get("memory_limit", ""),
                                    source=application.get("source", ""),
                                    create_time=application.get("create_time", "")
                                    )
            print json.dumps(application_info, indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def list_by_clusterid(auth, clusterid):
        try:
            data = App_base._list_by_clusterid(auth, clusterid)
            applications_info = []
            applications = data.get("applications", [])
            for application in applications:
                application_info = dict(id=application.get("id", ""),
                                        name=application.get("name", ""),
                                        lifecycle=application.get("lifecycle", ""),
                                        status_summary=application.get("status_summary", ""),
                                        running_status=application.get("running_status", ""),
                                        service_num=application.get("service_num", ""),
                                        task_num=application.get("task_num", ""),
                                        cpu_allocated=application.get("cpu_allocated", ""),
                                        cpu_limit=application.get("cpu_limit", ""),
                                        memory_allocated=application.get("memory_allocated", ""),
                                        memory_limit=application.get("memory_limit", ""),
                                        source=application.get("source", ""),
                                        create_time=application.get("create_time", "")
                                        )
                applications_info.append(application_info)
            print json.dumps(dict(applications=applications_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = App_base._statistics(auth)
            for cpu in data.get("ranks", {}).get("cpu", []):
                resource = cpu.get("resource_info", {})
                resource.pop("disk_allocated", "")
                resource.pop("disk_total", "")
            for memory in data.get("ranks", {}).get("memory", []):
                resource = memory.get("resource_info", {})
                resource.pop("disk_allocated", "")
                resource.pop("disk_total", "")
            print json.dumps(data, indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def entrance_list(auth, appid):
        try:
            data = App_base._entrance_list(auth, appid)
            entrances_info = []
            application_entrances = data.get("application_entrances", [])
            for application_entrance in application_entrances:
                entrance_info = dict(application_id=application_entrance.get("application_id", ""),
                                     service_id=application_entrance.get("service_id", ""),
                                     name=application_entrance.get("name", ""),
                                     url=application_entrance.get("url", "")
                                     )
                entrances_info.append(entrance_info)
            print json.dumps(dict(entrances=entrances_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def statistics_by_id(auth, appid):
        try:
            data = App_base._statistics_by_id(auth, appid)
            application = data[0] if len(data) > 0 else {}
            resource = application.get("resources", {})
            resource.pop("disk_allocated", "")
            resource.pop("disk_total", "")
            application.pop("application_id", "")
            print json.dumps(application, indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-


def args(*args, **kwargs):
    def _decorator(func):
        func.__dict__.setdefault('args', []).insert(0, (args, kwargs))
        return func

    return _decorator


def methods_of(obj):
    result = []
    for i in dir(obj):
        if callable(getattr(obj, i)) and not i.startswith('_'):
            result.append((i, getattr(obj, i), getattr(obj, i).__doc__))
    return result


def fetch_func_args(func, matchargs):
    fn_args = []
    for args, kwargs in getattr(func, 'args', []):
        arg = args[0]
        if arg.startswith("-"):
            arg = kwargs.get("dest")
        fn_args.append(getattr(matchargs, arg))

    return fn_args



# -*- coding: utf-8 -*-
import json
from ...utils.http_utils import http_request


class Service(object):
    @staticmethod
    def _list(auth, appid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}?application_id={}".format(security, address, "/dispatch/platform/v1/services", appid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _deploy_batch(auth, appid, serviceids):
        """服务上线"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/services/deploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=serviceids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # print (u"服务 {} 上线指令发送成功".format(','.join(serviceids)))
            print (u'Send "deploy {}" successfully'.format(','.join(serviceids)))
        except Exception as e:
            raise

    @staticmethod
    def _redeploy_batch(auth, appid, serviceids):
        """服务上线"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/services/redeploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=serviceids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # print (u"服务 {} 上线指令发送成功".format(','.join(serviceids)))
            print (u'Send "redeploy {}" successfully'.format(','.join(serviceids)))
        except Exception as e:
            raise

    @staticmethod
    def _undeploy_batch(auth, appid, serviceids):
        """服务下线"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/services/undeploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=serviceids,
                                       application_id=appid,
                                       delete_data=True))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u'Send "undeploy {}" successfully'.format(','.join(serviceids)))
        except Exception as e:
            raise

    @staticmethod
    def _delete_batch(auth, appid, serviceids):
        """批量删除服务"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/services")
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=serviceids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Delete service {} complete".format(','.join(serviceids)))
        except Exception as e:
            raise

    # @staticmethod
    # def delete(auth, serviceid):
    #     """删除服务"""
    #     print("delete service")



# -*- coding: utf-8 -*-
from .commonnode_ch import CommonNode as Node_ch
from .commonnode_en import CommonNode as Node_en
from .commonnode_json import CommonNode as Node_json
from .base.commonnode import CommonNode as Node_base


class CommonNodeDispatch(object):
    @staticmethod
    def list(auth, format_json):
        if format_json:
            Node_json.list(auth)
        else:
            if auth.language == "en":
                Node_en.list(auth)
            else:
                Node_ch.list(auth)

    @staticmethod
    def show(auth, machineid, format_json):
        if format_json:
            Node_json.show(auth, machineid)
        else:
            if auth.language == "en":
                Node_en.show(auth, machineid)
            else:
                Node_ch.show(auth, machineid)

    @staticmethod
    def list_by_clusterid(auth, clusterid, format_json):
        if format_json:
            Node_json.list_by_clusterid(auth, clusterid)
        else:
            if auth.language == "en":
                Node_en.list_by_clusterid(auth, clusterid)
            else:
                Node_ch.list_by_clusterid(auth, clusterid)

    @staticmethod
    def statistics(auth, format_json):
        if format_json:
            Node_json.statistics(auth)
        else:
            if auth.language == "en":
                Node_en.statistics(auth)
            else:
                Node_ch.statistics(auth)

    @staticmethod
    def deploy_batch(auth, clusterid, machineids):
        Node_base._deploy_batch(auth, clusterid, machineids)

    @staticmethod
    def undeploy_batch(auth, clusterid, machineids):
        Node_base._undeploy_batch(auth, clusterid, machineids)

    @staticmethod
    def delete_batch(auth, machineids):
        Node_base._delete_batch(auth, machineids)

    @staticmethod
    def add_batch(auth, clusterid, machineids, virtual_node_num):
        Node_base._add_batch(auth, clusterid, machineids, virtual_node_num)

    @staticmethod
    def remove_batch(auth, clusterid, machineids):
        Node_base._remove_batch(auth, clusterid, machineids)














# -*- coding: utf-8 -*-
from .cluster_ch import Cluster as Cluster_ch
from .cluster_en import Cluster as Cluster_en
from .cluster_json import Cluster as Cluster_json
from .base.cluster import Cluster as Cluster_base


class ClusterDispatch(object):
    @staticmethod
    def list(auth, format_json):
        if format_json:
            Cluster_json.list(auth)
        else:
            if auth.language == "en":
                Cluster_en.list(auth)
            else:
                Cluster_ch.list(auth)

    @staticmethod
    def show(auth, clusterid, format_json):
        if format_json:
            Cluster_json.show(auth, clusterid)
        else:
            if auth.language == "en":
                Cluster_en.show(auth, clusterid)
            else:
                Cluster_ch.show(auth, clusterid)

    @staticmethod
    def statistics_by_id(auth, clusterid, format_json):
        if format_json:
            Cluster_json.statistics_by_id(auth, clusterid)
        else:
            if auth.language == "en":
                Cluster_en.statistics_by_id(auth, clusterid)
            else:
                Cluster_ch.statistics_by_id(auth, clusterid)

    @staticmethod
    def statistics(auth, format_json):
        if format_json:
            Cluster_json.statistics(auth)
        else:
            if auth.language == "en":
                Cluster_en.statistics(auth)
            else:
                Cluster_ch.statistics(auth)

    @staticmethod
    def delete(auth, clusterid):
        Cluster_base._delete(auth, clusterid)










# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.commonnode import CommonNode as Node_base


occupied_conf_en = {True: "Occupied",
                    False: "Idle"}
lifecycle_conf_en = {"running": "Running",
                  "setting": "Setting",
                  "executing": "Executing"}
status_summary_conf_en = {"successfully_deployed": "Successfully deployed",
                       "failed_deployed": "Failed deployed",
                       "executing": "Executing",
                       "setting": "Setting"}
running_status_conf_en = {"green": "Normal",
                       "gray": "",
                       "yellow": "Warning",
                       "red": "Error"}


class CommonNode(object):
    @staticmethod
    def list(auth):
        try:
            data = Node_base._list(auth)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Hostname"
            header["occupied"] = "Occupied"
            header["ip"] = "Service IP"
            header["administration_ip"] = "Administration IP"
            header["username"] = "Username"
            header["ssh_port"] = "Ssh Port"
            header["cluster_id"] = "Cluster ID"
            header["cluster_name"] = "Cluster Name"
            header["data_dirs_new"] = "Data Dirs"
            header["create_time"] = "Create Time"
            nodes_info = []
            nodes = data.get("common_nodes", [])
            for node in nodes:
                node = obj_replace_time_field(node, ["create_time"])
                node["occupied"] = occupied_conf_en.get(node.get("occupied", None), "")
                data_dirs = node.get("data_dirs", [])
                node["data_dirs_new"] = "\n".join(
                    ["{}:{}".format(data_dir["name"], data_dir["path"]) for data_dir in data_dirs])
                node_info = [node.get(key, "") for key in header]
                nodes_info.append(node_info)
            # for node_info in nodes_info:
            #     data_dirs_tmp = node_info[header.keys().index("data_dirs")]
            #     # format
            #     # [{ "path": "/data","default": true,"name": "normal"},
            #     #  {"path": "/ssd","default": false,"name": "ssh"}]
            #     # to
            #     # "normal:/data\nssh:/ssd"
            #     data_dirs = "\n".join(
            #         ["{}:{}".format(data_dir["name"], data_dir["path"]) for data_dir in data_dirs_tmp])
            #     node_info[header.keys().index("data_dirs")] = data_dirs
            print_table(header.values(), nodes_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, machineid):
        try:
            data = Node_base._show(auth, machineid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Hostname"
            header["occupied"] = "Occupied"
            header["ip"] = "Service IP"
            header["administration_ip"] = "Administration IP"
            header["username"] = "Username"
            header["ssh_port"] = "Ssh Port"
            header["cluster_id"] = "Cluster ID"
            header["cluster_name"] = "Cluster Name"
            header["data_dirs_new"] = "Data Dirs"
            header["create_time"] = "Create Time"
            nodes_info = []
            nodes = data.get("common_nodes", [])
            for node in nodes:
                node = obj_replace_time_field(node, ["create_time"])
                node["occupied"] = occupied_conf_en.get(node.get("occupied", None), u"")
                data_dirs = node.get("data_dirs", [])
                node["data_dirs_new"] = "\n".join(
                    ["{}:{}".format(data_dir["name"], data_dir["path"]) for data_dir in data_dirs])
                node_info = [node.get(key, "") for key in header]
                nodes_info.append(node_info)
            # for node_info in nodes_info:
            #     data_dirs_tmp = node_info[header.keys().index("data_dirs")]
            #     # format
            #     # [{ "path": "/data","default": true,"name": "normal"},
            #     #  {"path": "/ssd","default": false,"name": "ssh"}]
            #     # to
            #     # "normal:/data\nssh:/ssd"
            #     data_dirs = "\n".join(
            #         ["{}:{}".format(data_dir["name"], data_dir["path"]) for data_dir in data_dirs_tmp])
            #     node_info[header.keys().index("data_dirs")] = data_dirs
            print_table(header.values(), nodes_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def list_by_clusterid(auth, clusterid):
        try:
            data1, data2 = Node_base._list_by_clusterid(auth, clusterid)
            usages = {}
            for _k, _v in data2.items():
                if not isinstance(_v, list):
                    continue
                for v in _v:
                    usages.setdefault(v.get("node_id", ""), {}).setdefault(_k, v.get("value", 0))
                    # usages[v.get("node_id", "")][_k] = v.get("value", 0)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Hostname"
            header["status_summary_running"] = "Status"
            header["ip"] = "Service IP"
            header["administration_ip"] = "Administration IP"
            header["username"] = "Username"
            header["ssh_port"] = "Ssh Port"
            header["cluster_id"] = "Cluster ID"
            header["cluster_name"] = "Cluster Name"
            header["data_dirs_new"] = "Data Dirs"
            header["cpu_usages"] = "CPU Usages（%）"
            header["disk_usages"] = "Disk Usages（%）"
            header["memory_usages"] = "Memory Usages（%）"
            header["create_time"] = "Create Time"
            nodes_info = []
            nodes = data1.get("nodes", [])
            for node in nodes:
                node = obj_replace_time_field(node, ["create_time"])
                status_summary_running = status_format(node.get("lifecycle", ""), lifecycle_conf_en,
                                                       node.get("status_summary", ""), status_summary_conf_en,
                                                       node.get("running_status", ""), running_status_conf_en)
                node["status_summary_running"] = status_summary_running
                node.update(usages.get(node.get("id", ""), {}))
                data_dirs = node.get("data_dirs", [])
                node["data_dirs_new"] = "\n".join(
                    ["{}:{}".format(data_dir["name"], data_dir["path"]) for data_dir in data_dirs])
                node_info = [node.get(key, "") for key in header]
                nodes_info.append(node_info)
            # for node_info in nodes_info:
            #     data_dirs_tmp = node_info[header.keys().index("data_dirs")]
            #     # format
            #     # [{ "path": "/data","default": true,"name": "normal"},
            #     #  {"path": "/ssd","default": false,"name": "ssh"}]
            #     # to
            #     # "normal:/data\nssh:/ssd"
            #     data_dirs = "\n".join(
            #         ["{}:{}".format(data_dir["name"], data_dir["path"]) for data_dir in data_dirs_tmp])
            #     node_info[header.keys().index("data_dirs")] = data_dirs
            print_table(header.values(), nodes_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = Node_base._statistics(auth)
            header = collections.OrderedDict()
            header["total"] = "Total host num"
            header["occupied"] = "Occupied num"
            header["disk"] = "Total Disk(G)"
            header["memory"] = "Total Memory(M)"
            header["cpu"] = "Total CPU(Cors)"
            nodes_info = []
            nodes = data.get("common_nodes", {})
            nodes.update(data.get("resources", {}))
            node_info = [nodes.get(key, "") for key in header]
            nodes_info.append(node_info)
            print_table(header.values(), nodes_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.file import File as File_base


class File(object):
    @staticmethod
    def list(auth, path):
        try:
            data = File_base._list(auth, path)
            header = collections.OrderedDict()
            header["name"] = u"名称"
            header["path"] = u"路径"
            header["size"] = u"大小(kB)"
            header["type"] = u"类型"
            header["date"] = u"创建时间"
            files_info = []
            files = data.get("result", [])
            for file in files:
                file["size"] = round(file.get("size", 0)/1024.0, 2)
                file_info = [file.get(key, "") for key in header]
                files_info.append(file_info)
            print_table(header.values(), files_info, auth.encoding)
        except Exception as e:
            raise


# -*- coding: utf-8 -*-
import time
import datetime
from prettytable import PrettyTable, FRAME, ALL, NONE
import sys
import tarfile


def get_item_bins(item_list, bin_num_max=1):
    assert item_list and isinstance(item_list, list)
    item_bin_list = []
    bin_num = min(len(item_list), bin_num_max)
    for _ in range(0, bin_num):
        item_bin_list.append([])
    for index in range(len(item_list)):
        item_bin_index = index % bin_num
        item_bin_list[item_bin_index].append(item_list[index])
    return item_bin_list


def utc2local(utc_st):
    """UTC时间转本地时间（+8:00）"""
    now_stamp = time.time()
    local_time = datetime.datetime.fromtimestamp(now_stamp)
    utc_time = datetime.datetime.utcfromtimestamp(now_stamp)
    offset = local_time - utc_time
    local_st = utc_st + offset
    return local_st


def local2utc(local_st):
    """本地时间转UTC时间（-8:00）"""
    time_struct = time.mktime(local_st.timetuple())
    utc_st = datetime.datetime.utcfromtimestamp(time_struct)
    return utc_st


def print_table(header, data_list, encoding):
    try:
        # print sys.stdout.encoding
        x = PrettyTable(list(header), border=False, encoding=encoding)
        # x = PrettyTable(list(header), border=False, encoding=sys.stdout.encoding)
        x.padding_width = 1
        x.hrules = ALL
        for data in data_list:
            x.add_row(list(data))
        print x
    except Exception as e:
        print e.message


def timestamp_to_datetime(int_time):
    timeStamp = int_time
    timeArray = time.localtime(timeStamp)
    otherStyleTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
    return otherStyleTime


def obj_replace_time_field(oobj, field_name_list):
    assert isinstance(oobj, dict)
    for field_name in field_name_list:
        oobj_time = oobj.get(field_name, None)
        if oobj_time and (isinstance(oobj_time, int) or oobj_time.isdigit()):
            oobj_datetime = timestamp_to_datetime(oobj_time)
            oobj[field_name] = oobj_datetime
    return oobj


def status_format(lifecycle, lifecycle_conf, status_summary, status_summary_conf, running_status, running_status_conf, type=None):
    assert isinstance(lifecycle_conf, dict)
    assert isinstance(status_summary_conf, dict)
    assert isinstance(running_status_conf, dict)
    if type == "task":
        status = status_format_task(lifecycle, lifecycle_conf, running_status, running_status_conf)
        return status
    if type == "application":
        status = status_format_app(status_summary, status_summary_conf, running_status, running_status_conf)
        return status
    _lifecycle = lifecycle_conf.get(lifecycle, "")
    _status_summary = status_summary_conf.get(status_summary, "")
    _running_status = running_status_conf.get(running_status, "")
    if lifecycle == "running":
        if status_summary == "failed_deployed":
            status = _status_summary
        else:
            status = u"{}[{}]".format(_status_summary, _running_status)
    else:
        status = _lifecycle
    return status


def status_format_app(status_summary, status_summary_conf, running_status, running_status_conf):
    assert isinstance(status_summary_conf, dict)
    assert isinstance(running_status_conf, dict)
    _status_summary = status_summary_conf.get(status_summary, "")
    _running_status = running_status_conf.get(running_status, "")
    if status_summary == "successfully_deployed":
        status = u"{}[{}]".format(_status_summary, _running_status)
    else:
        status = _status_summary
    return status


def status_format_task(lifecycle, lifecycle_conf, running_status, running_status_conf):
    assert isinstance(lifecycle_conf, dict)
    assert isinstance(running_status_conf, dict)
    _lifecycle = lifecycle_conf.get(lifecycle, "")
    _running_status = running_status_conf.get(running_status, "")
    if lifecycle == "scheduling":
        status = u"{}[{}]".format(_lifecycle, _running_status)
    else:
        status = _lifecycle
    return status


def extract(tar_path, target_path):
    try:
        tar = tarfile.open(tar_path, "r")
        tar.extractall(target_path)
        tar.close()
    except Exception, e:
        raise Exception, e



# -*- coding: utf-8 -*-
from .microservice_ch import MicroService as Ms_ch
from .microservice_en import MicroService as Ms_en
from .microservice_json import MicroService as Ms_json
from .base.microservice import MicroService as Ms_base


class MicroServiceDispatch(object):
    @staticmethod
    def list(auth, serviceid, format_json):
        """根据服务ID显示该服务下微服务列表"""
        if format_json:
            Ms_json.list(auth, serviceid)
        else:
            if auth.language == "en":
                Ms_en.list(auth, serviceid)
            else:
                Ms_ch.list(auth, serviceid)

    @staticmethod
    def statistics(auth, serviceid, format_json):
        if format_json:
            Ms_json.statistics(auth, serviceid)
        else:
            if auth.language == "en":
                Ms_en.statistics(auth, serviceid)
            else:
                Ms_ch.statistics(auth, serviceid)

    @staticmethod
    def delete_batch(auth, serviceid, microserviceids):
        """批量删除微服务"""
        Ms_base._delete_batch(auth, serviceid, microserviceids)



# -*- coding: utf-8 -*-
import json
from ...utils.http_utils import http_request


class MicroService(object):
    @staticmethod
    def _list(auth, serviceid):
        """根据服务ID显示该服务下微服务列表"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}?edit_mode=false&service_id={}".format(security, address, "/dispatch/platform/v1/micro_services", serviceid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _statistics(auth, serviceid):
        """根据服务ID统计该服务下微服务信息"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}?edit_mode=False&service_id={}".format(security, address, "/dispatch/platform/v1/micro_services",
                                                                 serviceid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _delete_batch(auth, serviceid, microserviceids):
        """批量删除微服务"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/micro_services")
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=microserviceids,
                                       service_id=serviceid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Delete micro service {} complete".format(','.join(microserviceids)))
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
import json
from ...utils.http_utils import http_request


class CommonNode(object):
    @staticmethod
    def _list(auth):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/common_nodes")
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _show(auth, machineid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        machineid = [machineid]
        url = "{}://{}{}{}".format(security, address, "/dispatch/platform/v1/common_nodes?ids=", ",".join(machineid))
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _list_by_clusterid(auth, clusterid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url1 = "{}://{}{}?cluster_id={}".format(security, address, "/dispatch/platform/v1/nodes", clusterid)
        url2 = "{}://{}{}?cluster_id={}".format(security, address, "/dispatch/nodemonitor/v1/nodes", clusterid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url1, http_method=http_method, headers=headers)
        data1 = json.loads(response_data)
        if status != 200:
            message = data1.get("message", "Unknown Error")
            raise Exception(message)
        status, response_data = http_request(url=url2, http_method=http_method, headers=headers)
        data2 = json.loads(response_data)
        if status != 200:
            message = data2.get("message", "Unknown Error")
            raise Exception(message)
        return data1, data2

    @staticmethod
    def _statistics(auth):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/statistics?type=common_node")
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _deploy_batch(auth, clusterid, machineids):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/nodes/deploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=machineids,
                                       cluster_id=clusterid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"主机 {} 上线命令发送成功".format(','.join(machineids)))
        except Exception as e:
            raise

    @staticmethod
    def _undeploy_batch(auth, clusterid, machineids):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/nodes/undeploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=machineids,
                                       cluster_id=clusterid,
                                       delete_data=True))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"主机 {} 下线命令发送成功".format(','.join(machineids)))
        except Exception as e:
            raise

    @staticmethod
    def _delete_batch(auth, machineids):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/common_nodes")
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=machineids))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"主机 {} 删除成功".format(','.join(machineids)))
        except Exception as e:
            raise

    @staticmethod
    def _add_batch(auth, clusterid, machineids, virtual_node_num):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/nodes")
        http_method = "POST"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(cluster_id=clusterid,
                                       common_nodes=
                                       [dict(common_node_id=machineid,
                                             virtual_node_num=virtual_node_num) for machineid in machineids]
                                       ))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"添加主机 {} 到集群 {} 成功".format(','.join(machineids), clusterid))
        except Exception as e:
            raise

    @staticmethod
    def _remove_batch(auth, clusterid, machineids):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/nodes")
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=machineids,
                                       cluster_id=clusterid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"从集群 {} 移除主机 {} 成功".format(clusterid, ','.join(machineids)))
        except Exception as e:
            raise














# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.cluster import ClusterDispatch as Cluster
from .v3_0.commonnode import CommonNodeDispatch as CN
from .v3_0.application import ApplicationDispatch as Application
from .v3_0.service import ServiceDispatch as Service
from .v3_0.task import TaskDispatch as Task
from .v3_0.microservice import MicroServiceDispatch as MicroService
from .v3_0.instance import InstanceDispatch as Instance
from .v3_0.file import FileDispatch as File


class LsCommands(object):
    @args("--clusterid", dest="clusterid", default=None, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def cluster(self, auth, clusterid=None, format_json=False):
        """list clusters info"""
        if clusterid:
            Cluster.show(auth, clusterid, format_json)
        else:
            Cluster.list(auth, format_json)

    @args("--machineid", dest="machineid", default=None, help="")
    @args("--clusterid", dest="clusterid", default=None, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def machine(self, auth, machineid=None, clusterid=None, format_json=False):
        """list machines info"""
        if not machineid and not clusterid:
            CN.list(auth, format_json)
        elif not clusterid:
            CN.show(auth, machineid, format_json)
        elif not machineid:
            CN.list_by_clusterid(auth, clusterid, format_json)
        else:
            raise Exception("Parameter error")

    @args("--appid", dest="appid", default=None, help="")
    @args("--clusterid", dest="clusterid", default=None, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def application(self, auth, appid=None, clusterid=None, format_json=False):
        """list applications info"""
        if not appid and not clusterid:
            Application.list(auth, format_json)
        elif not clusterid:
            Application.show(auth, appid, format_json)
        elif not appid:
            Application.list_by_clusterid(auth, clusterid, format_json)
        else:
            raise Exception("Parameter error")

    @args("--appid", dest="appid", required=True, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def service(self, auth, appid, format_json=False):
        """list services info"""
        Service.list(auth, appid, format_json)

    @args("--appid", dest="appid", required=True, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def task(self, auth, appid, format_json=False):
        """list tasks info"""
        Task.list(auth, appid, format_json)

    @args("--serviceid", dest="serviceid", required=True, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def microservice(self, auth, serviceid, format_json=False):
        """list micro services info"""
        MicroService.list(auth, serviceid, format_json)

    @args("--serviceid", dest="serviceid", required=True, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def instance(self, auth, serviceid, format_json=False):
        """list instances info"""
        Instance.list(auth, serviceid, format_json)

    @args("--appid", dest="appid", required=True, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def entrance(self, auth, appid, format_json=False):
        """list application's entrances info"""
        Application.entrance_list(auth, appid, format_json)

    @args("--path", dest="path", default="/", help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def file(self, auth, path="/", format_json=False):
        """list files on file manager"""
        File.list(auth, path, format_json)

    # @args("--serviceid", dest="serviceid", required=True, help=u"实例对应的服务id")
    # @args("--instanceid", dest="instanceid", help=u"实例id")
    # @args("-t", dest="terminal_flag", default=False, action='store_true', help=u"开启终端,必须输入instanceid及serviceid")
    # @args("-l", dest="log_flag", default=False, action='store_true', help=u"查看日志,必须输入instanceid及serviceid")
    # def instance(self, auth, serviceid, instanceid, terminal_flag, log_flag):
    #     """显示实例列表"""
    #     if terminal_flag and log_flag:
    #         raise Exception("Parameter error")
    #     elif not terminal_flag and not log_flag:
    #         Instance.list(auth, serviceid)
    #     elif not log_flag:
    #         Instance.terminal(auth, serviceid, instanceid)
    #     else:
    #         Instance.log(auth, serviceid, instanceid)


    # @args("--serviceid", dest="serviceid", required=True, help=u"根据服务id查询实例")
    # @args("--instanceid", dest="instanceid", required=True, help=u"根据实例id查询log")
    # def instance_log(self, auth, servieid, instanceid):
    #     """显示实例日志"""
    #     Instance.log(auth, servieid, instanceid)
    #
    # @args("--serviceid", dest="serviceid", required=True, help=u"根据服务id查询实例")
    # @args("--instanceid", dest="instanceid", required=True, help=u"根据实例id查询log")
    # def instance_terminal(self, auth, servieid, instanceid):
    #     """显示实例日志"""
    #     Instance.terminal(auth, servieid, instanceid)


# -*- coding: utf-8 -*-
from .application_ch import Application as App_ch
from .application_en import Application as App_en
from .application_json import Application as App_json
from .base.application import Application as App_base


class ApplicationDispatch(object):
    @staticmethod
    def list(auth, format_json):
        if format_json:
            App_json.list(auth)
        else:
            if auth.language == "en":
                App_en.list(auth)
            else:
                App_ch.list(auth)

    @staticmethod
    def show(auth, appid, format_json):
        if format_json:
            App_json.show(auth, appid)
        else:
            if auth.language == "en":
                App_en.show(auth, appid)
            else:
                App_ch.show(auth, appid)

    @staticmethod
    def list_by_clusterid(auth, clusterid, format_json):
        if format_json:
            App_json.list_by_clusterid(auth, clusterid)
        else:
            if auth.language == "en":
                App_en.list_by_clusterid(auth, clusterid)
            else:
                App_ch.list_by_clusterid(auth, clusterid)

    @staticmethod
    def statistics(auth, format_json):
        if format_json:
            App_json.statistics(auth)
        else:
            if auth.language == "en":
                App_en.statistics(auth)
            else:
                App_ch.statistics(auth)

    @staticmethod
    def entrance_list(auth, appid, format_json):
        if format_json:
            App_json.entrance_list(auth, appid)
        else:
            if auth.language == "en":
                App_en.entrance_list(auth, appid)
            else:
                App_ch.entrance_list(auth, appid)

    @staticmethod
    def statistics_by_id(auth, appid, format_json):
        if format_json:
            App_json.statistics_by_id(auth, appid)
        else:
            if auth.language == "en":
                App_en.statistics_by_id(auth, appid)
            else:
                App_ch.statistics_by_id(auth, appid)

    @staticmethod
    def delete(auth, appid):
        App_base._delete(auth, appid)



# -*- coding: utf-8 -*-
import json
import os
import random
from ...utils.http_utils import http_request
from requests_toolbelt.multipart.encoder import MultipartEncoder


class File(object):
    @staticmethod
    def _list(auth, path):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        request_body = json.dumps(dict(action="list",
                                       path=path))
        status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _upload(auth, dest, files):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/fileservice/v1/file")
        http_method = "POST"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token}
        file_name = "file"
        body_dict = {}
        for index, _file in enumerate(files):
            if os.access(_file, os.R_OK):
                body_dict["{}-{}".format(file_name, index)] = (os.path.basename(_file), open(_file, 'rb'), 'application/octet-stream')
            else:
                raise Exception("File Error")
        body_dict["destination"] = dest
        multipart_encoder = MultipartEncoder(
            fields=body_dict
        )
        headers['Content-Type'] = multipart_encoder.content_type
        status, response_data = http_request(url=url, http_method=http_method, body=multipart_encoder, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        result = data.get("result",{})
        if result.get("success"):
            print "Uploade files:{} success".format(",".join(files))
        else:
            print result.get("error")

    @staticmethod
    def _download(auth, path):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        filename = os.path.basename(path)
        url = "{}://{}{}?action=download&path={}&Authorization={}&X-Access-Authority={}".format(
            security, address, "/dispatch/fileservice/v1/file", path, auth_token, access_authority_token)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, content = http_request(url=url, http_method=http_method, headers=headers)
        if status == 200:
            with open(filename, 'wb') as f:
                f.write(content)
        print "success"



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.task import Task as Task_base


lifecycle_conf_ch = {"scheduling": u"调度中",
                     "setting": u"配置中"}
running_status_conf_ch = {"idle": u"未运行",
                          "running": u"运行中",
                          "success": u"正常",
                          "failed": u"失败",
                          "killed": u"中止"}


class Task(object):
    @staticmethod
    def list(auth, appid):
        try:
            data = Task_base._list(auth, appid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"任务名"
            header["status_summary_running"] = u"当前状态"
            header["application_id"] = u"应用ID"
            header["cluster_id"] = u"集群ID"
            header["cluster_name"] = u"集群名"
            header["version"] = u"版本"
            header["prototype"] = u"类型"
            header["cpu_allocated_total"] = u"CPU使用率（核）"
            header["memory_allocated_total"] = u"内存使用率（M）"
            header["schedule_start"] = u"任务开始时间"
            header["schedule_interval"] = u"任务间隔时间"
            header["create_time"] = u"创建时间"
            tasks_info = []
            tasks = data.get("tasks", [])
            for task in tasks:
                task["cpu_allocated_total"] = "{}/{}".format(
                    task.get("resource_info", {}).get("cpu_allocated", "NaN"),
                    task.get("resource_info", {}).get("cpu_total", "NaN"))
                task["memory_allocated_total"] = "{}/{}".format(
                    task.get("resource_info", {}).get("memory_allocated", "NaN"),
                    task.get("resource_info", {}).get("memory_total", "NaN"))
                task["schedule_start"] = task.get("schedule", {}).get("start_time", "")
                interval = task.get("schedule", {}).get("interval", {})
                task["schedule_interval"] = "{}{}".format(interval.get("value", ""), interval.get("unit", ""))
                task = obj_replace_time_field(task, ["create_time"])
                status_summary_running = status_format(task.get("lifecycle", ""), lifecycle_conf_ch,
                                                       "", dict(),
                                                       task.get("running_status", ""), running_status_conf_ch,
                                                       "task")
                task["status_summary_running"] = status_summary_running
                task_info = [task.get(key, "") for key in header]
                tasks_info.append(task_info)
            print_table(header.values(), tasks_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.commonnode import CommonNodeDispatch as CN
from .v3_0.instance import InstanceDispatch as Instance


class AddCommands(object):
    @args("--virtual_node_num", dest="virtual_node_num", default=1, help=u"虚拟节点数")
    @args("--machineid", dest="machineids", required=True, nargs="+", help=u"主机ID")
    @args("--clusterid", dest="clusterid", required=True, help=u"所属集群ID")
    def machine(self, auth, virtual_node_num, machineids, clusterid):
        """add machine to cluster"""
        CN.add_batch(auth, clusterid, machineids, virtual_node_num)

    @args("--instanceid", dest="instanceids", required=True, nargs="+", help=u"实例ID")
    @args("--microserviceid", dest="microserviceuid", required=True, help=u"所属微服务ID")
    @args("--serviceid", dest="serviceid", required=True, help=u"所属服务ID")
    def instance(self, auth, instanceids, microserviceid, serviceid):
        """add instance"""
        Instance.add_batch(auth, serviceid, microserviceid, instanceids)


# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.cluster import Cluster as Cluster_base


lifecycle_conf_en = {"running": "RUNNING",
                  "setting": "SETTING",
                  "executing": "EXECUTING"}
status_summary_conf_en = {
    "setting": "SETTING",
    "successfully_deployed": "SUCCESSFULLY DEPLOYED",
    "failed_deployed": "FAILED DEPLOYED",
    "executing": "EXECUTING"
}
running_status_conf_en = {
    "green": "Normal",
    "yellow": "Warning",
    "red": "Error"
}


class Cluster(object):
    @staticmethod
    def list(auth):
        try:
            data = Cluster_base._list(auth)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Cluster Name"
            header["status_summary_running"] = "Status"
            header["application_num"] = "Application Num"
            header["creater_name"] = "Creator"
            header["create_time"] = "Create Time"
            clusters_info = []
            clusters = data.get("clusters", [])
            users = data.get("users", {})
            for cluster in clusters:
                cluster = obj_replace_time_field(cluster, ["create_time"])
                cluster["status_summary_running"] = status_format(cluster.get("lifecycle", ""), lifecycle_conf_en,
                                                                  cluster.get("status_summary", ""),
                                                                  status_summary_conf_en,
                                                                  cluster.get("running_status", ""),
                                                                  running_status_conf_en)
                cluster["creater_name"] = users.get(cluster.get("creater", ""), {}).get("nickname", "") \
                                          or users.get(cluster.get("creater", ""), {}).get("username", "")
                cluster_info = [cluster.get(key, "") for key in header]
                clusters_info.append(cluster_info)
            print_table(header.values(), clusters_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, clusterid):
        try:
            data = Cluster_base._show(auth, clusterid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Cluster Name"
            header["status_summary_running"] = "Status"
            header["create_time"] = "Create Time"
            clusters_info = []
            cluster = data
            cluster = obj_replace_time_field(cluster, ["create_time"])
            cluster["status_summary_running"] = status_format(cluster.get("lifecycle", ""), lifecycle_conf_en,
                                                              cluster.get("status_summary", ""), status_summary_conf_en,
                                                              cluster.get("running_status", ""), running_status_conf_en)
            cluster_info = [cluster.get(key, "") for key in header]
            clusters_info.append(cluster_info)
            print_table(header.values(), clusters_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics_by_id(auth, clusterid):
        try:
            data1, data2 = Cluster_base._statistics_by_id(auth, clusterid)
            header = collections.OrderedDict()
            header["cluster_id"] = "ID"
            header["cluster_name"] = "Cluster name"
            header["total"] = "Total Node"
            header["running"] = "Running Node"
            header["executing"] = "Executing Node"
            header["setting"] = "Setting Node"
            header["cpu_total"] = "Total CPU"
            header["cpu_used"] = "Used CPU"
            header["cpu_usage"] = "CPU Usage(%)"
            header["disk_total"] = "Total Disk(G)"
            header["disk_used"] = "Used Disk"
            header["disk_usage"] = "Disk Usage(%)"
            header["memory_total"] = "Total Memory(M)"
            header["memory_used"] = "Used Memory"
            header["memory_usage"] = "Memory Usage(%)"
            clusters_info = []
            clusters = data1
            for cluster in clusters:
                nodes = cluster.get("nodes", {})
                resources = cluster.get("resources", {})
                cluster.update(nodes)
                cluster.update(resources)
                cluster.update(data2)
                cluster["cpu_used"] = int(cluster.get("cpu_used", 0))
                cluster["cpu_total"] = int(cluster.get("cpu_total", 0))
                cluster_info = [cluster.get(key, "") for key in header]
                clusters_info.append(cluster_info)
            print_table(header.values(), clusters_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = Cluster_base._statistics(auth)
            print "Application Status"
            header1 = collections.OrderedDict()
            header1["error"] = "Error"
            header1["running"] = "Running"
            header1["setting"] = "Setting"
            header1["executing"] = "Executing"
            header1["total"] = "Total"
            counts_info = []
            count = data.get("count", {})
            count_info = [count.get(key, "") for key in header1]
            counts_info.append(count_info)
            print_table(header1.values(), counts_info, auth.encoding)
            print "CPU usages(cor)："
            header2 = collections.OrderedDict()
            header2["cluster_id"] = "Cluster id"
            header2["cluster_name"] = "Cluster name"
            header2["cpu_used"] = "Used CPU"
            header2["cpu_total"] = "Total CPU"
            cpus_info = []
            cpus = data.get("ranks", {}).get("cpu", [])
            for cpu in cpus:
                cpu["cpu_used"] = int(cpu.get("cpu_used", 0))
                cpu["cpu_total"] = int(cpu.get("cpu_total", 0))
                cpu_info = [cpu.get(key, "") for key in header2]
                cpus_info.append(cpu_info)
            print_table(header2.values(), cpus_info, auth.encoding)
            print "Memory usages(M)："
            header3 = collections.OrderedDict()
            header3["cluster_id"] = "Cluster id"
            header3["cluster_name"] = "Cluster name"
            header3["memory_used"] = "Used memory"
            header3["memory_total"] = "Total memory"
            memorys_info = []
            memorys = data.get("ranks", {}).get("memory", [])
            for memory in memorys:
                memory_info = [memory.get(key, "") for key in header3]
                memorys_info.append(memory_info)
            print_table(header3.values(), memorys_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.instance import Instance as Ins_base


class Instance(object):
    @staticmethod
    def list(auth, serviceid):
        try:
            data = Ins_base._list(auth, serviceid)
            instances_info = []
            microservices = data.get("micro_services", [])
            for microservice in microservices:
                for instance in microservice.get("instances", []):
                    instance_info = dict(id=instance.get("id", ""),
                                         name=instance.get("name", ""),
                                         lifecycle=instance.get("lifecycle", ""),
                                         status_summary=instance.get("status_summary", ""),
                                         running_status=instance.get("running_status", ""),
                                         micro_service_id=instance.get("name", ""),
                                         service_id=instance.get("service_id", ""),
                                         application_id=instance.get("application_id", ""),
                                         cluster_id=instance.get("cluster_id", ""),
                                         node_id=instance.get("node_id", ""),
                                         running_node_id=instance.get("running_node_id", ""),
                                         node_name=instance.get("node_name", ""),
                                         running_node_name=instance.get("running_node_name", ""),
                                         data_path_instance=instance.get("data_path_instance", ""),
                                         create_time=instance.get("create_time", ""),
                                         )
                    instances_info.append(instance_info)
            print json.dumps(dict(instances=instances_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from ..utils.common_utils import obj_replace_time_field
from .base.cluster import Cluster as Cluster_base


class Cluster(object):
    @staticmethod
    def list(auth):
        try:
            data = Cluster_base._list(auth)
            clusters_info = []
            clusters = data.get("clusters", [])
            users = data.get("users", {})
            for cluster in clusters:
                cluster_info = dict(
                    id=cluster.get("id", ""),
                    name=cluster.get("name", ""),
                    lifecycle_conf=cluster.get("lifecycle_conf", ""),
                    status_summary=cluster.get("status_summary", ""),
                    running_status=cluster.get("running_status", ""),
                    cluster_name=users.get(cluster.get("creater", ""), {}).get("nickname", "")
                                 or users.get(cluster.get("creater", ""), {}).get("username", ""),
                    create_time=cluster.get("create_time", "")
                )
                clusters_info.append(cluster_info)
            print json.dumps(dict(clusters=clusters_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, clusterid):
        try:
            data = Cluster_base._show(auth, clusterid)
            cluster_info = {}
            cluster = data
            cluster_info["id"] = cluster.get("id", "")
            cluster_info["name"] = cluster.get("name", "")
            cluster_info["lifecycle"] = cluster.get("lifecycle", "")
            cluster_info["status_summary"] = cluster.get("status_summary", "")
            cluster_info["running_status"] = cluster.get("running_status", "")
            cluster_info["create_time"] = cluster.get("create_time", "")
            print json.dumps(cluster_info, indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def statistics_by_id(auth, clusterid):
        try:
            data1, data2 = Cluster_base._statistics_by_id(auth, clusterid)
            cluster = data1[0] if len(data1) > 0 else {}
            if cluster:
                cluster["monitor"] = data2
            print json.dumps(cluster, indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = Cluster_base._statistics(auth)
            print json.dumps(data, indent=4, ensure_ascii=False)
        except Exception as e:
            raise


# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.file import File as File_base


class File(object):
    @staticmethod
    def list(auth):
        try:
            data = File_base._list(auth)
            header = collections.OrderedDict()
            header["name"] = "Name"
            header["path"] = "Path"
            header["size"] = "Size(kB)"
            header["type"] = "Type"
            header["date"] = "Date"
            files_info = []
            files = data.get("result", [])
            for file in files:
                file["size"] = round(file.get("size", 0) / 1024.0, 2)
                file_info = [file.get(key, "") for key in header]
                files_info.append(file_info)
            print_table(header.values(), files_info, auth.encoding)
        except Exception as e:
            raise


# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.service import ServiceDispatch as Service


class RedeployCommands(object):
    @args("--serviceid", dest="serviceids", required=True, nargs="+", help="")
    @args("--appid", dest="appid", required=True, help="")
    def service(self, auth, serviceids, appid):
        """redeploy service"""
        Service.redeploy_batch(auth, appid, serviceids)





# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.task import TaskDispatch as Task


class StartCommands(object):
    @args("--taskid", dest="taskids", required=True, nargs="+", help="")
    @args("--appid", dest="appid", required=True, help="")
    def task(self, auth, taskids, appid):
        """start task"""
        Task.start_batch(auth, appid, taskids)



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.service import Service as Srv_base


lifecycle_conf_en = {"running": "Running",
                     "setting": "Setting",
                     "executing": "Executing"}
status_summary_conf_en = {"successfully_deployed": "Successfully deployed",
                          "failed_deployed": "Failed deployed",
                          "executing": "Executing",
                          "setting": "Setting"}
running_status_conf_en = {"green": "Normal",
                          "gray": "",
                          "yellow": "Warning",
                          "red": "Error"}


class Service(object):
    @staticmethod
    def list(auth, appid):
        try:
            data = Srv_base._list(auth, appid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Name"
            header["status_summary_running"] = "Status"
            header["application_id"] = "Application ID"
            header["cluster_id"] = "Cluster ID"
            header["cluster_name"] = "Cluster Name"
            header["prototype"] = "Prototype"
            header["version"] = "Version"
            header["micro_service_num"] = "Micro Service Num"
            header["cpu_allocated_total"] = "Used CPU(cor)"
            header["memory_allocated_total"] = "Used Memory(M)"
            header["create_time"] = "Create Time"
            services_info = []
            services = data.get("services", [])
            for service in services:
                service["cpu_allocated_total"] = "{}/{}".format(
                    service.get("resource_info", {}).get("cpu_allocated", "NaN"),
                    service.get("resource_info", {}).get("cpu_total", "NaN"))
                service["memory_allocated_total"] = "{}/{}".format(
                    service.get("resource_info", {}).get("memory_allocated", "NaN"),
                    service.get("resource_info", {}).get("memory_total", "NaN"))
                service = obj_replace_time_field(service, ["create_time"])
                status_summary_running = status_format(service.get("lifecycle", ""), lifecycle_conf_en,
                                                       service.get("status_summary", ""), status_summary_conf_en,
                                                       service.get("running_status", ""), running_status_conf_en)
                service["status_summary_running"] = status_summary_running
                service_info = [service.get(key, "") for key in header]
                services_info.append(service_info)
            print_table(header.values(), services_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
import os


def get_ttyclient_uri():
    ttyclient_uri = os.getenv("ADMINISTRATION_ENV_TTYCLIENT_URI", "http://su.huayun.com/resserverpub/bin/bin.tar")
    return ttyclient_uri




# -*- coding: utf-8 -*-
import json
from ...utils.http_utils import http_request


class Cluster(object):
    @staticmethod
    def _list(auth):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/misc/v1/clusters")
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _show(auth, clusterid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}/{}".format(security, address, "/dispatch/platform/v1/clusters", clusterid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _statistics_by_id(auth, clusterid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url1 = "{}://{}{}?type=cluster&ids={}".format(security, address, "/dispatch/platform/v1/statistics", clusterid)
        url2 = "{}://{}{}/{}".format(security, address, "/dispatch/nodemonitor/v1/clusters", clusterid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url1, http_method=http_method, headers=headers)
        data1 = json.loads(response_data)
        if status != 200:
            message = data1.get("message", "Unknown Error")
            raise Exception(message)
        status, response_data = http_request(url=url2, http_method=http_method, headers=headers)
        data2 = json.loads(response_data)
        if status != 200:
            message = data2.get("message", "Unknown Error")
            raise Exception(message)
        return data1, data2

    @staticmethod
    def _statistics(auth):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/cluster_statistics")
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _delete(auth, clusterid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}/{}".format(security, address, "/dispatch/platform/v1/clusters", clusterid)
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        response_data = json.dumps(dict())
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=response_data, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Delete cluster {} complete".format(clusterid))
        except Exception as e:
            raise










# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.file import File as File_base


class File(object):
    @staticmethod
    def list(auth, path):
        try:
            data = File_base._list(auth, path)
            files = data.get("result", [])
            print json.dumps(dict(files=files), indent=4, ensure_ascii=False)
        except Exception as e:
            raise


# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.commonnode import CommonNodeDispatch as CN


class RemoveCommands(object):
    @args("--machineid", dest="machineids", required=True, nargs="+", help="")
    @args("--clusterid", dest="clusterid", required=True, help="")
    def machine(self, auth, machineids, clusterid):
        """remove machine from cluster"""
        CN.remove_batch(auth, clusterid, machineids)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.commonnode import CommonNode as Node_base


class CommonNode(object):
    @staticmethod
    def list(auth):
        try:
            data = Node_base._list(auth)
            nodes_info = []
            nodes = data.get("common_nodes", [])
            for node in nodes:
                node_info = dict(id=node.get("id", ""),
                                 name=node.get("name", ""),
                                 occupied=node.get("occupied", ""),
                                 ip=node.get("ip", ""),
                                 administration_ip=node.get("administration_ip"),
                                 username=node.get("username", ""),
                                 ssh_port=node.get("ssh_port", ""),
                                 cluster_id=node.get("cluster_id", ""),
                                 cluster_name=node.get("cluster_name", ""),
                                 data_dirs=node.get("data_dirs", []),
                                 create_time=node.get("create_time", "")
                                 )
                nodes_info.append(node_info)
            # print json.dumps(nodes_info)
            print json.dumps(dict(nodes=nodes_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, machineid):
        try:
            data = Node_base._show(auth, machineid)
            nodes_info = []
            nodes = data.get("common_nodes", [])
            for node in nodes:
                node_info = dict(id=node.get("id", ""),
                                 name=node.get("name", ""),
                                 occupied=node.get("occupied", ""),
                                 ip=node.get("ip", ""),
                                 administration_ip=node.get("administration_ip", ""),
                                 username=node.get("username", ""),
                                 ssh_port=node.get("ssh_port", ""),
                                 cluster_id=node.get("cluster_id", ""),
                                 cluster_name=node.get("cluster_name", ""),
                                 data_dirs=node.get("data_dirs", ""),
                                 create_time=node.get("create_time", "")
                                 )
                nodes_info.append(node_info)
            print json.dumps(nodes_info[0] if len(nodes_info) > 0 else dict(), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def list_by_clusterid(auth, clusterid):
        try:
            data1, data2 = Node_base._list_by_clusterid(auth, clusterid)
            usages = {}
            for _k, _v in data2.items():
                if not isinstance(_v, list):
                    continue
                for v in _v:
                    usages.setdefault(v.get("node_id", ""), {}).setdefault(_k, v.get("value", 0))
                    # usages[v.get("node_id", "")][_k] = v.get("value", 0)
            nodes_info = []
            nodes = data1.get("nodes", [])
            for node in nodes:
                node.update(usages.get(node.get("id", ""), {}))
                node_info = dict(id=node.get("id", ""),
                                 name=node.get("name", ""),
                                 lifecycle=node.get("lifecycle", ""),
                                 status_summary=node.get("status_summary", ""),
                                 running_status=node.get("running_status", ""),
                                 ip=node.get("ip", ""),
                                 administration_ip=node.get("administration_ip", ""),
                                 username=node.get("username", ""),
                                 ssh_port=node.get("ssh_port", ""),
                                 cluster_id=node.get("cluster_id", ""),
                                 cluster_name=node.get("cluster_name", ""),
                                 data_dirs=node.get("data_dirs", ""),
                                 cpu_usages=node.get("cpu_usages", ""),
                                 disk_usages=node.get("disk_usages", ""),
                                 memory_usages=node.get("memory_usages", ""),
                                 create_time=node.get("create_time", "")
                                 )
                nodes_info.append(node_info)
            print json.dumps(dict(nodes=nodes_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = Node_base._statistics(auth)
            node = data.get("common_nodes", {})
            node.update(data.get("resources", {}))
            node_info = dict(total=node.get("total", ""),
                             occupied=node.get("occupied", ""),
                             disk=node.get("disk", ""),
                             memory=node.get("memory", ""),
                             cpu=node.get("cpu", "")
                             )
            print json.dumps(node_info, indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.file import FileDispatch as File


class DownloadCommands(object):
    @args("--path", dest="path", required=True, help="")
    def file(self, auth, path):
        """download file"""
        File.download(auth, path)



# -*- coding: utf-8 -*-
from .service_ch import Service as Srv_ch
from .service_en import Service as Srv_en
from .service_json import Service as Srv_json
from .base.service import Service as Srv_base


class ServiceDispatch(object):
    @staticmethod
    def list(auth, appid, format_json):
        """根据应用ID显示该应用下服务列表"""
        if format_json:
            Srv_json.list(auth, appid)
        else:
            if auth.language == "en":
                Srv_en.list(auth, appid)
            else:
                Srv_ch.list(auth, appid)

    @staticmethod
    def deploy_batch(auth, appid, serviceids):
        """服务上线"""
        Srv_base._deploy_batch(auth, appid, serviceids)

    @staticmethod
    def redeploy_batch(auth, appid, serviceids):
        """服务上线"""
        Srv_base._redeploy_batch(auth, appid, serviceids)

    @staticmethod
    def undeploy_batch(auth, appid, serviceids):
        """服务下线"""
        Srv_base._undeploy_batch(auth, appid, serviceids)

    @staticmethod
    def delete_batch(auth, appid, serviceids):
        """批量删除服务"""
        Srv_base._delete_batch(auth, appid, serviceids)

    # @staticmethod
    # def delete(auth, serviceid):
    #     """删除服务"""
    #     print("delete service")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.service import Service as Srv_base


class Service(object):
    @staticmethod
    def list(auth, appid):
        try:
            data = Srv_base._list(auth, appid)
            services_info = []
            services = data.get("services", [])
            for service in services:
                service_info = dict(id=service.get("id", ""),
                                    name=service.get("name", ""),
                                    lifecycle=service.get("lifecycle", ""),
                                    status_summary=service.get("status_summary", ""),
                                    running_status=service.get("running_status", ""),
                                    application_id=service.get("application_id", ""),
                                    cluster_id=service.get("cluster_id", ""),
                                    cluster_name=service.get("cluster_name", ""),
                                    prototype=service.get("prototype", ""),
                                    version=service.get("version", ""),
                                    micro_service_num=service.get("micro_service_num", ""),
                                    cpu_allocated_total=service.get("cpu_allocated_total", ""),
                                    memory_allocated_total=service.get("memory_allocated_total", ""),
                                    create_time=service.get("create_time", ""),
                                    )
                services_info.append(service_info)
            print json.dumps(dict(services=services_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.cluster import ClusterDispatch as Cluster
from .v3_0.commonnode import CommonNodeDispatch as CN
from .v3_0.application import ApplicationDispatch as Application
from .v3_0.microservice import MicroServiceDispatch as MicroService


class StatisCommands(object):
    @args("--clusterid", dest="clusterid", default=None, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def cluster(self, auth, clusterid=None, format_json=False):
        """statistics info of clusters"""
        if clusterid:
            Cluster.statistics_by_id(auth, clusterid, format_json)
        else:
            Cluster.statistics(auth, format_json)

    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def machine(self, auth, format_json=False):
        """statistics info of machines"""
        CN.statistics(auth, format_json)

    @args("--appid", dest="appid", default=None, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def application(self, auth, appid=None, format_json=False):
        """statistics info of applications"""
        if appid:
            Application.statistics_by_id(auth, appid, format_json)
        else:
            Application.statistics(auth, format_json)

    @args("--serviceid", dest="serviceid", required=True, help="")
    @args("-f", dest="format_json", default=False, action='store_true', help="return data in JSON format")
    def microservice(self, auth, serviceid, format_json=False):
        """statistics info of micro services"""
        MicroService.statistics(auth, serviceid, format_json)




# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.commonnode import CommonNodeDispatch as CN
from .v3_0.service import ServiceDispatch as Service
from .v3_0.task import TaskDispatch as Task
from .v3_0.instance import InstanceDispatch as Instance


class DeployCommands(object):
    @args("--machineid", dest="machineids", required=True, nargs="+", help="")
    @args("--clusterid", dest="clusterid", required=True, help="")
    def machine(self, auth, machineids, clusterid):
        """deploy machine"""
        CN.deploy_batch(auth, clusterid, machineids)

    @args("--serviceid", dest="serviceids", required=True, nargs="+", help="")
    @args("--appid", dest="appid", required=True, help="")
    def service(self, auth, serviceids, appid):
        """deploy service"""
        Service.deploy_batch(auth, appid, serviceids)

    @args("--instanceid", dest="instanceids", required=True, nargs="+", help="")
    @args("--microserviceid", dest="microserviceid", required=True, help="")
    @args("--serviceid", dest="serviceid", required=True, help="")
    def instance(self, auth, instanceids, microserviceid, serviceid):
        """deploy instance"""
        Instance.deploy_batch(auth, serviceid, microserviceid, instanceids)

    @args("--taskid", dest="taskids", required=True, nargs="+", help="")
    @args("--appid", dest="appid", required=True, help="")
    def task(self, auth, taskids, appid):
        """deploy task"""
        Task.deploy_batch(auth, appid, taskids)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
import collections
from .base.microservice import MicroService as Ms_base


class MicroService(object):
    @staticmethod
    def list(auth, serviceid):
        try:
            data = Ms_base._list(auth, serviceid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Microservice Name"
            header["status_summary_running"] = "Status"
            header["service_id"] = "Service ID"
            header["application_id"] = "Application ID"
            header["cluster_id"] = "Cluster ID"
            header["prototype"] = "Service Type"
            header["instance_num"] = "Instance Num"
            header["version"] = "Version"
            header["cpu"] = "CPU(cor)"
            header["memory"] = "Memory(M)"
            header["create_time"] = "Create Time"
            microservices_info = []
            microservices = data.get("micro_services", [])
            for microservice in microservices:
                microservice_info = dict(id=microservice.get("id", ""),
                                         name=microservice.get("name", ""),
                                         lifecycle=microservice.get("lifecycle", ""),
                                         status_summary=microservice.get("status_summary", ""),
                                         running_status=microservice.get("running_status", ""),
                                         service_id=microservice.get("service_id", ""),
                                         application_id=microservice.get("application_id", ""),
                                         cluster_id=microservice.get("cluster_id", ""),
                                         prototype=microservice.get("prototype", ""),
                                         instance_num=microservice.get("instance_num", ""),
                                         version=microservice.get("version", ""),
                                         cpu=microservice.get("cpu", ""),
                                         memory=microservice.get("memory", ""),
                                         create_time=microservice.get("create_time", ""),
                                         )
                microservices_info.append(microservice_info)
            print json.dumps(dict(micro_services=microservices_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth, serviceid):
        try:
            data = Ms_base._statistics(auth, serviceid)
            statistics = data.get("statistics", {})
            print json.dumps(statistics, indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.service import Service as Srv_base


lifecycle_conf_ch = {"running": u"已上线",
                     "setting": u"配置中",
                     "executing": u"执行中"}
status_summary_conf_ch = {"successfully_deployed": u"已上线",
                          "failed_deployed": u"上线失败",
                          "executing": u"执行中",
                          "setting": u"配置中"}
running_status_conf_ch = {"green": u"正常",
                          "gray": u"",
                          "yellow": u"异常",
                          "red": u"错误"}


class Service(object):
    @staticmethod
    def list(auth, appid):
        try:
            data = Srv_base._list(auth, appid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"服务名"
            header["status_summary_running"] = u"当前状态"
            header["application_id"] = u"应用ID"
            header["cluster_id"] = u"集群ID"
            header["cluster_name"] = u"集群名"
            header["prototype"] = u"类型"
            header["version"] = u"版本"
            header["micro_service_num"] = u"微服务数"
            header["cpu_allocated_total"] = u"CPU使用率（核）"
            header["memory_allocated_total"] = u"内存使用率（M）"
            header["create_time"] = u"创建时间"
            services_info = []
            services = data.get("services", [])
            for service in services:
                service["cpu_allocated_total"] = "{}/{}".format(
                    service.get("resource_info", {}).get("cpu_allocated", "NaN"),
                    service.get("resource_info", {}).get("cpu_total", "NaN"))
                service["memory_allocated_total"] = "{}/{}".format(
                    service.get("resource_info", {}).get("memory_allocated", "NaN"),
                    service.get("resource_info", {}).get("memory_total", "NaN"))
                service = obj_replace_time_field(service, ["create_time"])
                status_summary_running = status_format(service.get("lifecycle", ""), lifecycle_conf_ch,
                                                       service.get("status_summary", ""), status_summary_conf_ch,
                                                       service.get("running_status", ""), running_status_conf_ch)
                service["status_summary_running"] = status_summary_running
                service_info = [service.get(key, "") for key in header]
                services_info.append(service_info)
            print_table(header.values(), services_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.application import Application as App_base


status_summary_conf_ch = {"successfully_deployed": u"已上线",
                       "failed_deployed": u"上线失败",
                       "executing": u"执行中",
                       "setting": u"配置中"}
running_status_conf_ch = {"green": u"正常",
                          "gray": u"",
                          "yellow": u"异常",
                          "red": u"错误"}
source_conf_ch = {"du": u"普通应用",
               "saas": u"SaaS应用"}


class Application(object):
    @staticmethod
    def list(auth):
        try:
            data = App_base._list(auth)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"应用名称"
            header["status_summary_running"] = u"当前状态"
            header["cpu_limit"] = u"CPU(核)"
            header["memory_limit"] = u"内存(M)"
            header["service_num"] = u"服务数"
            header["service_running_num"] = u"运行中服务"
            header["task_num"] = u"任务数"
            header["task_running_num"] = u"运行中任务"
            header["source"] = u"来源"
            header["creater"] = u"创建人"
            header["create_time"] = u"创建时间"
            applications_info = []
            applications = data.get("applications", [])
            users = data.get("users", {})
            for application in applications:
                application = obj_replace_time_field(application, ["create_time"])
                application["status_summary_running"] = status_format(application.get("lifecycle", ""),
                                                                      {},
                                                                      application.get("status_summary", ""),
                                                                      status_summary_conf_ch,
                                                                      application.get("running_status", ""),
                                                                      running_status_conf_ch,
                                                                      "application")
                application["source"] = source_conf_ch.get(application.get("source", ""), u"")
                application_info = [application.get(key, "") for key in header]
                creater_name = users.get(application.get("creater", ""), {}).get("nickname", "") \
                               or users.get(application.get("creater", ""), {}).get("username", "")
                application_info[header.keys().index("creater")] = creater_name
                applications_info.append(application_info)
            print_table(header.values(), applications_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, appid):
        try:
            data = App_base._show(auth, appid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"应用名称"
            header["status_summary_running"] = u"当前状态"
            header["cpu_limit"] = u"CPU(核)"
            header["memory_limit"] = u"内存(M)"
            header["source"] = u"来源"
            header["create_time"] = u"创建时间"
            applications_info = []
            application = data
            application = obj_replace_time_field(application, ["create_time"])
            application["status_summary_running"] = status_format(application.get("lifecycle", ""),
                                                                  {},
                                                                  application.get("status_summary", ""),
                                                                  status_summary_conf_ch,
                                                                  application.get("running_status", ""),
                                                                  running_status_conf_ch,
                                                                  "application")
            application["source"] = source_conf_ch.get(application.get("source", ""), u"")
            application_info = [application.get(key, "") for key in header]
            applications_info.append(application_info)
            print_table(header.values(), applications_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def list_by_clusterid(auth, clusterid):
        try:
            data = App_base._list_by_clusterid(auth, clusterid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"应用名称"
            header["status_summary_running"] = u"当前状态"
            header["service_num"] = u"服务(个)"
            header["task_num"] = u"任务(个)"
            header["cpu_allocated"] = u"分配CPU(核)"
            header["cpu_limit"] = u"总CPU(核)"
            header["memory_allocated"] = u"分配内存(M)"
            header["memory_limit"] = u"总内存(M)"
            header["source"] = u"来源"
            header["create_time"] = u"创建时间"
            applications_info = []
            applications = data.get("applications", [])
            for application in applications:
                application = obj_replace_time_field(application, ["create_time"])
                application["status_summary_running"] = status_format(application.get("lifecycle", ""),
                                                                  {},
                                                                  application.get("status_summary", ""),
                                                                  status_summary_conf_ch,
                                                                  application.get("running_status", ""),
                                                                  running_status_conf_ch,
                                                                  "application")
                application["source"] = source_conf_ch.get(application.get("source", ""), u"")
                application_info = [application.get(key, "") for key in header]
                applications_info.append(application_info)
            print_table(header.values(), applications_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = App_base._statistics(auth)
            print u"应用状态："
            header1 = collections.OrderedDict()
            header1["du"] = u"普通应用"
            header1["saas"] = u"SaaS应用"
            header1["error"] = u"异常"
            header1["running"] = u"运行中"
            header1["setting"] = u"配置中"
            header1["total"] = u"总数"
            counts_info = []
            count = data.get("count", {})
            count_info = [count.get(key, "") for key in header1]
            counts_info.append(count_info)
            print_table(header1.values(), counts_info, auth.encoding)
            print u"各应用CPU使用情况(核)："
            header2 = collections.OrderedDict()
            header2["id"] = u"ID"
            header2["name"] = u"应用名称"
            header2["cpu_allocated"] = u"CPU已分配"
            header2["cpu_total"] = u"CPU总数"
            cpus_info = []
            cpus = data.get("ranks", {}).get("cpu", [])
            for cpu in cpus:
                cpu.update(cpu.get("resource_info", {}))
                cpu_info = [cpu.get(key, "") for key in header2]
                cpus_info.append(cpu_info)
            print_table(header2.values(), cpus_info, auth.encoding)
            print u"各应用内存使用情况(M)："
            header3 = collections.OrderedDict()
            header3["id"] = u"ID"
            header3["name"] = u"应用名称"
            header3["memory_allocated"] = u"内存已分配"
            header3["memory_total"] = u"内存总数"
            memorys_info = []
            memorys = data.get("ranks", {}).get("memory", [])
            for memory in memorys:
                memory.update(memory.get("resource_info", {}))
                memory_info = [memory.get(key, "") for key in header3]
                memorys_info.append(memory_info)
            print_table(header3.values(), memorys_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def entrance_list(auth, appid):
        try:
            data = App_base._entrance_list(auth, appid)
            header = collections.OrderedDict()
            header["application_id"] = u"应用ID"
            header["service_id"] = u"服务ID"
            header["name"] = u"应用入口"
            header["url"] = u"访问地址"
            entrances_info = []
            application_entrances = data.get("application_entrances", [])
            for application_entrance in application_entrances:
                entrance_info = [application_entrance.get(key, "") for key in header]
                entrances_info.append(entrance_info)
            print_table(header.values(), entrances_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics_by_id(auth, appid):
        try:
            data = App_base._statistics_by_id(auth, appid)
            print u"资源状态："
            header1 = collections.OrderedDict()
            header1["application_id"] = u"ID"
            header1["cpu_allocated"] = u"CPU已分配（核）"
            header1["cpu_total"] = u"CPU总数"
            header1["memory_allocated"] = u"内存已分配（M）"
            header1["memory_total"] = u"内存总量"
            resources_info = []
            apps = data
            for app in apps:
                app.update(app.get("resources", {}))
                resource_info = [app.get(key, "") for key in header1]
                resources_info.append(resource_info)
            print_table(header1.values(), resources_info, auth.encoding)
            print u"服务运行情况："
            header2 = collections.OrderedDict()
            header2["running"] = u"已上线"
            header2["setting"] = u"配置中"
            header2["total"] = u"总数"
            services_info = []
            for app in apps:
                app.update(app.get("services", {}))
                service_info = [app.get(key, "") for key in header2]
                services_info.append(service_info)
            print_table(header2.values(), services_info, auth.encoding)
            print u"任务运行情况："
            header3 = collections.OrderedDict()
            header3["running"] = u"已上线"
            header3["setting"] = u"配置中"
            header3["total"] = u"总数"
            tasks_info = []
            for app in apps:
                app.update(app.get("tasks", {}))
                task_info = [app.get(key, "") for key in header3]
                tasks_info.append(task_info)
            print_table(header3.values(), tasks_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from ..utils.http_utils import http_request
from ..utils.env_utils import get_ttyclient_uri
from ..utils.common_utils import extract
import json
import ConfigParser
import os
import sys
import requests


class HttpClient(object):

    def __init__(self, loginfile=".serviceultra-auth"):
        if sys.platform == "linux2":
            self.os_verify = "/etc/serviceultractl/os-verify"
            self.login_conf = os.path.join(os.environ['HOME'], loginfile)
            self.ttyclient = os.path.dirname(os.path.realpath(__file__)) + "/../bin/" + "ttyclient-linux"
            self.tarpath = os.path.dirname(os.path.realpath(__file__)) + "/../"
            self.encoding = "UTF-8"
        elif sys.platform == "win32":
            self.os_verify = "C:\\\\serviceultractl\\os-verify"
            self.login_conf = loginfile
            self.ttyclient = os.path.dirname(os.path.realpath(__file__)) + "\\..\\bin\\" + "ttyclient-win.exe"
            self.tarpath = os.path.dirname(os.path.realpath(__file__)) + "\\..\\"
            self.encoding = "GBK"
        else:
            raise Exception("The operating system is not supported.")
        self.ttyclient_check()
        self.auth_token = None
        self.access_authority_token = None
        self.address = None
        self.security = None
        self.language = None

    def get_token(self):
        try:
            # 生成config对象
            conf = ConfigParser.ConfigParser()
            # 用config对象读取配置文件
            conf.read(self.login_conf)
            if not conf.has_section("tokens"):
                raise Exception("Please login")
            self.auth_token = conf.get("tokens", "auth_token")
            self.access_authority_token = conf.get("tokens", "access_authority_token")
            self.address = conf.get("tokens", "address")
            self.security = conf.get("tokens", "security")
            self.language = conf.get("tokens", "language")
        except Exception as e:
            raise Exception("Illegal token.Please login again")

    def login(self, verify_conf=None):
        if verify_conf:
            username = verify_conf.get("username")
            password = verify_conf.get("password")
            domain = verify_conf.get("domain")
            address = verify_conf.get("address")
            security = verify_conf.get("security")
            language = verify_conf.get("language")
        else:
            username, password, domain, address, security, language = self.get_verify_info()
        if address.startswith("http"):
            url = "{}{}".format(address, "/dispatch/login/v1/login")
            security = address.split("://")[0]
            address = address.split("://")[-1]
        else:
            url = "{}://{}{}".format(security, address, "/dispatch/login/v1/login")
        http_method = "POST"
        request_body = json.dumps(dict(username=username,
                            password=password,
                            domain=domain))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body,
                                                            headers={"Content-Type": "application/json"})
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # 生成config对象
            conf = ConfigParser.ConfigParser()
            # 用config对象读取配置文件
            conf.read(self.login_conf)
            if not conf.has_section("tokens"):
                # 增加新的section
                conf.add_section('tokens')
            conf.set('tokens', 'auth_token', data.get("auth_token"))
            conf.set('tokens', 'access_authority_token', data.get("access_authority_token"))
            conf.set('tokens', 'address', address)
            conf.set('tokens', 'security', security)
            conf.set('tokens', 'language', language)
            with open(self.login_conf, 'w') as fw:  # 循环写入
                conf.write(fw)
            # print u"用户:{} 登录成功".format(username)
            print "USER:{} LOGIN".format(username)
        except Exception as e:
            raise

    def logout(self):
        # 生成config对象
        conf = ConfigParser.ConfigParser()
        # 用config对象读取配置文件
        conf.read(self.login_conf)
        if not conf.has_section("tokens"):
            # 增加新的section
            conf.add_section('tokens')
        conf.remove_option('tokens', 'auth_token')
        conf.remove_option('tokens', 'access_authority_token')
        conf.remove_option('tokens', 'address')
        conf.remove_option('tokens', 'security')
        conf.remove_option('tokens','language')
        with open(self.login_conf, 'w') as fw:  # 循环写入
            conf.write(fw)
        print "Logout"

    def get_short_token(self):
        auth_token = self.auth_token or ''
        access_authority_token = self.access_authority_token or ''
        address = self.address or ''
        security = self.security or ''
        url = "{}://{}{}".format(security, address, "/dispatch/authority/v1/shortToken")
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token, "Content-Type": "application/json"}
        try:
            status, response_data = http_request(url=url, http_method=http_method, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            return data.get("auth_token"), data.get("access_authority_token")
        except Exception as e:
            raise

    def get_verify_info(self):
        try:
            # 生成config对象
            conf = ConfigParser.ConfigParser()
            # 用config对象读取配置文件
            conf.read(self.os_verify)
            username = conf.get("verify", "username")
            password = conf.get("verify", "password")
            domain = conf.get("verify", "domain")
            address = conf.get("verify", "address")
            security = conf.get("verify", "security")
            language = conf.get("verify", "language")
            return username, password, domain, address, security, language
        except Exception as e:
            raise Exception("Please login")

    def ttyclient_check(self):
        if not os.path.isfile(self.ttyclient):
            print "Downloading ttyclient......"
            download_uri = get_ttyclient_uri()
            file_name = os.path.basename(download_uri)
            r = requests.get(download_uri)
            with open(self.tarpath+file_name, "wb") as code:
                code.write(r.content)
            extract(self.tarpath+file_name, self.tarpath)
            print "Ttyclient downloaded"


# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.application import Application as App_base


status_summary_conf_en = {"successfully_deployed": "Successfully deployed",
                          "failed_deployed": "Failed deployed",
                          "executing": "Executing",
                          "setting": "Setting"}
running_status_conf_en = {"green": "Normal",
                          "gray": "",
                          "yellow": "Warning",
                          "red": "Error"}
source_conf_en = {"du": "Su",
                  "saas": "SaaS"}


class Application(object):
    @staticmethod
    def list(auth):
        try:
            data = App_base._list(auth)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Name"
            header["status_summary_running"] = "Status"
            header["cpu_limit"] = "CPU limit(Cor)"
            header["memory_limit"] = "Memory limit(M)"
            header["service_num"] = "Service Num"
            header["service_running_num"] = "Running Service Num"
            header["task_num"] = "Task Num"
            header["task_running_num"] = "Running Task Num"
            header["source"] = "Source"
            header["creater_name"] = "Creator"
            header["create_time"] = "Create Time"
            applications_info = []
            applications = data.get("applications", [])
            users = data.get("users", {})
            for application in applications:
                application = obj_replace_time_field(application, ["create_time"])
                application["status_summary_running"] = status_format(application.get("lifecycle", ""),
                                                                      {},
                                                                      application.get("status_summary", ""),
                                                                      status_summary_conf_en,
                                                                      application.get("running_status", ""),
                                                                      running_status_conf_en,
                                                                      "application")
                application["source"] = source_conf_en.get(application.get("source", ""), u"")
                application["creater_name"] = users.get(application.get("creater", ""), {}).get("nickname", "") \
                                          or users.get(application.get("creater", ""), {}).get("username", "")
                application_info = [application.get(key, "") for key in header]
                applications_info.append(application_info)
            print_table(header.values(), applications_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def show(auth, appid):
        try:
            data = App_base._show(auth, appid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Name"
            header["status_summary_running"] = "Status"
            header["cpu_limit"] = "CPU limit(Cor)"
            header["memory_limit"] = "Memory limit(M)"
            header["source"] = "Source"
            header["create_time"] = "Create Time"
            applications_info = []
            application = data
            application = obj_replace_time_field(application, ["create_time"])
            application["status_summary_running"] = status_format(application.get("lifecycle", ""),
                                                                  {},
                                                                  application.get("status_summary", ""),
                                                                  status_summary_conf_en,
                                                                  application.get("running_status", ""),
                                                                  running_status_conf_en,
                                                                  "application")
            application["source"] = source_conf_en.get(application.get("source", ""), u"")
            application_info = [application.get(key, "") for key in header]
            applications_info.append(application_info)
            print_table(header.values(), applications_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def list_by_clusterid(auth, clusterid):
        try:
            data = App_base._list_by_clusterid(auth, clusterid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Name"
            header["status_summary_running"] = "Status"
            header["service_num"] = "Service Num"
            header["task_num"] = "Task Num"
            header["cpu_allocated"] = "Allocated CPU"
            header["cpu_limit"] = "Total CPU"
            header["memory_allocated"] = "Allocated Memory(M)"
            header["memory_limit"] = "Total memory(M)"
            header["source"] = "Source"
            header["create_time"] = "Create Time"
            applications_info = []
            applications = data.get("applications", [])
            for application in applications:
                application = obj_replace_time_field(application, ["create_time"])
                application["status_summary_running"] = status_format(application.get("lifecycle", ""),
                                                                      {},
                                                                      application.get("status_summary", ""),
                                                                      status_summary_conf_en,
                                                                      application.get("running_status", ""),
                                                                      running_status_conf_en,
                                                                      "application")
                application["source"] = source_conf_en.get(application.get("source", ""), u"")
                application_info = [application.get(key, "") for key in header]
                applications_info.append(application_info)
            print_table(header.values(), applications_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth):
        try:
            data = App_base._statistics(auth)
            print "Applications Status:"
            header1 = collections.OrderedDict()
            header1["du"] = "Common Application"
            header1["saas"] = "SaaS Application"
            header1["error"] = "Error"
            header1["running"] = "Running"
            header1["setting"] = "Setting"
            header1["total"] = "Total"
            counts_info = []
            count = data.get("count", {})
            count_info = [count.get(key, "") for key in header1]
            counts_info.append(count_info)
            print_table(header1.values(), counts_info, auth.encoding)
            print u"Rank of CPU usages(cor):"
            header2 = collections.OrderedDict()
            header2["id"] = "ID"
            header2["name"] = "Name"
            header2["cpu_allocated"] = "Allocated CPU"
            header2["cpu_total"] = "Total CPU"
            cpus_info = []
            cpus = data.get("ranks", {}).get("cpu", [])
            for cpu in cpus:
                cpu.update(cpu.get("resource_info", {}))
                cpu_info = [cpu.get(key, "") for key in header2]
                cpus_info.append(cpu_info)
            print_table(header2.values(), cpus_info, auth.encoding)
            print u"Rank of Memory usages(M):"
            header3 = collections.OrderedDict()
            header3["id"] = "ID"
            header3["name"] = "Name"
            header3["memory_allocated"] = "Allocated Memory"
            header3["memory_total"] = "Total Memory"
            memorys_info = []
            memorys = data.get("ranks", {}).get("memory", [])
            for memory in memorys:
                memory.update(memory.get("resource_info", {}))
                memory_info = [memory.get(key, "") for key in header3]
                memorys_info.append(memory_info)
            print_table(header3.values(), memorys_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def entrance_list(auth, appid):
        try:
            data = App_base._entrance_list(auth, appid)
            header = collections.OrderedDict()
            header["application_id"] = "Application ID"
            header["service_id"] = "Service ID"
            header["name"] = "Entrance Name"
            header["url"] = "Url"
            entrances_info = []
            application_entrances = data.get("application_entrances", [])
            for application_entrance in application_entrances:
                entrance_info = [application_entrance.get(key, "") for key in header]
                entrances_info.append(entrance_info)
            print_table(header.values(), entrances_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics_by_id(auth, appid):
        try:
            data = App_base._statistics_by_id(auth, appid)
            print "Resource Status:"
            header1 = collections.OrderedDict()
            header1["application_id"] = "Application ID"
            header1["cpu_allocated"] = "Allocated CPU(cor)"
            header1["cpu_total"] = "Total CPU"
            header1["memory_allocated"] = "Allocated Memory(M)"
            header1["memory_total"] = "Total Memory"
            resources_info = []
            apps = data
            for app in apps:
                app.update(app.get("resources", {}))
                resource_info = [app.get(key, "") for key in header1]
                resources_info.append(resource_info)
            print_table(header1.values(), resources_info, auth.encoding)
            print "Service Running Status:"
            header2 = collections.OrderedDict()
            header2["running"] = "Running"
            header2["setting"] = "Setting"
            header2["total"] = "Total"
            services_info = []
            for app in apps:
                app.update(app.get("services", {}))
                service_info = [app.get(key, "") for key in header2]
                services_info.append(service_info)
            print_table(header2.values(), services_info, auth.encoding)
            print u"Task Running Status:"
            header3 = collections.OrderedDict()
            header3["running"] = "Running"
            header3["setting"] = "Setting"
            header3["total"] = "Total"
            tasks_info = []
            for app in apps:
                app.update(app.get("tasks", {}))
                task_info = [app.get(key, "") for key in header3]
                tasks_info.append(task_info)
            print_table(header3.values(), tasks_info, auth.encoding)
        except Exception as e:
            raise




# -*- coding: utf-8 -*-
import json
import os
from ...utils.http_utils import http_request


class Instance(object):
    @staticmethod
    def _list(auth, serviceid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}?edit_mode=false&service_id={}".format(security, address,
                                                               "/dispatch/platform/v1/micro_services", serviceid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _show(auth, instanceid):
        """查询单个实例"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}/{}".format(security, address, "/dispatch/platform/v1/instances", instanceid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        try:
            status, response_data = http_request(url=url, http_method=http_method, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            return data
        except Exception as e:
            raise

    @staticmethod
    def _deploy_batch(auth, serviceid, microserviceid, instanceids):
        """部署实例"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/instances/deploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=instanceids,
                                       micro_service_id=microserviceid,
                                       service_id=serviceid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"实例 {} 部署指令发送成功".format(','.join(instanceids)))
        except Exception as e:
            raise

    @staticmethod
    def _undeploy_batch(auth, serviceid, microserviceid, instanceids):
        """卸载实例"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/instances/undeploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=instanceids,
                                       micro_service_id=microserviceid,
                                       service_id=serviceid,
                                       delete_data=False))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"实例 {} 卸载指令发送成功".format(','.join(instanceids)))
        except Exception as e:
            raise

    @staticmethod
    def _delete_batch(auth, serviceid, microserviceid, instanceids):
        """批量删除实例"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/instances")
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=instanceids,
                                       micro_service_id=microserviceid,
                                       service_id=serviceid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Delete instance {} complete".format(','.join(instanceids)))
        except Exception as e:
            raise

    @staticmethod
    def _add_batch(auth, serviceid, microserviceid, instanceids):
        """批量添加实例"""
        print(u"功能暂未开放")

    @staticmethod
    def _log(auth, instanceid):
        """实例日志"""
        short_auth_token, short_access_authority_token = auth.get_short_token()
        data = Instance._show(auth, instanceid)
        domain_name_service = data.get("domain_name_service", "")
        if not domain_name_service:
            raise Exception("Unknown Error")
        wsurl = "http://{}/webtty/{}:59998/{}/{}/?ins_id={}".format(auth.address, domain_name_service, short_auth_token,
                                                             short_access_authority_token, instanceid)
        try:
            cmd = "{} {}".format(auth.ttyclient, wsurl)
            os.system(cmd)
        except KeyboardInterrupt:
            exit(0)
        except Exception:
            raise

    @staticmethod
    def _terminal(auth, instanceid):
        """实例终端"""
        short_auth_token, short_access_authority_token = auth.get_short_token()
        data = Instance._show(auth, instanceid)
        domain_name_service = data.get("domain_name_service", "")
        if not domain_name_service:
            raise Exception("Unknown Error")
        wsurl = "http://{}/webtty/{}:59999/{}/{}/?ins_id={}".format(auth.address, domain_name_service, short_auth_token,
                                                                   short_access_authority_token, instanceid)
        try:
            cmd = "{} {}".format(auth.ttyclient, wsurl)
            os.system(cmd)
        except KeyboardInterrupt:
            exit(0)
        except Exception:
            raise


# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.microservice import MicroService as Ms_base


lifecycle_conf_ch = {"running": u"已上线",
                  "setting": u"配置中",
                  "executing": u"执行中"}
status_summary_conf_ch = {"successfully_deployed": u"已上线",
                       "failed_deployed": u"上线失败",
                       "executing": u"执行中",
                       "setting": u"配置中"}
running_status_conf_ch = {"green": u"正常",
                       "gray": u"",
                       "yellow": u"异常",
                       "red": u"错误"}


class MicroService(object):
    @staticmethod
    def list(auth, serviceid):
        try:
            data = Ms_base._list(auth, serviceid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = u"微服务名"
            header["status_summary_running"] = u"当前状态"
            header["service_id"] = u"服务ID"
            header["application_id"] = u"应用ID"
            header["cluster_id"] = u"集群ID"
            header["prototype"] = u"服务类型"
            header["instance_num"] = u"实例数"
            header["version"] = u"版本"
            header["cpu"] = u"CPU（核）"
            header["memory"] = u"内存（M）"
            header["create_time"] = u"创建时间"
            microservices_info = []
            microservices = data.get("micro_services", [])
            for microservice in microservices:
                microservice = obj_replace_time_field(microservice, ["create_time"])
                status_summary_running = status_format(microservice.get("lifecycle", ""), lifecycle_conf_ch,
                                                       microservice.get("status_summary", ""), status_summary_conf_ch,
                                                       microservice.get("running_status", ""), running_status_conf_ch)
                microservice["status_summary_running"] = status_summary_running
                microservice_info = [microservice.get(key, "") for key in header]
                microservices_info.append(microservice_info)
            print_table(header.values(), microservices_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth, serviceid):
        try:
            data = Ms_base._statistics(auth, serviceid)
            header = collections.OrderedDict()
            header["cpu"] = u"CPU（核）"
            header["instance_num"] = u"实例个数"
            header["memory"] = u"内存（M）"
            header["micro_service_num"] = u"微服务数"
            statistics_infos = []
            statistics = data.get("statistics", {})
            statistics_info = [statistics.get(key, "") for key in header]
            statistics_infos.append(statistics_info)
            print_table(header.values(), statistics_infos, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
import json
from ...utils.http_utils import http_request


class Task(object):
    @staticmethod
    def _list(auth, appid):
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}?application_id={}".format(security, address, "/dispatch/platform/v1/tasks", appid)
        http_method = "GET"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        status, response_data = http_request(url=url, http_method=http_method, headers=headers)
        data = json.loads(response_data)
        if status != 200:
            message = data.get("message", "Unknown Error")
            raise Exception(message)
        return data

    @staticmethod
    def _deploy_batch(auth, appid, taskids):
        """任务上线"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/tasks/deploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=taskids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # print (u"任务 {} 上线指令发送成功".format(','.join(taskids)))
            print (u"Command:deploy {} sent successfully".format(','.join(taskids)))
        except Exception as e:
            raise

    @staticmethod
    def _undeploy_batch(auth, appid, taskids):
        """任务下线"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/tasks/undeploy")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=taskids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # print (u"任务 {} 下线指令发送成功".format(','.join(taskids)))
            print (u"Command:undeploy {} sent successfully".format(','.join(taskids)))
        except Exception as e:
            raise

    @staticmethod
    def _start_batch(auth, appid, taskids):
        """启动任务"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/tasks/start")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=taskids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # print (u"任务 {} 启动指令发送成功".format(','.join(taskids)))
            print (u"Command:start {} sent successfully".format(','.join(taskids)))
        except Exception as e:
            raise

    @staticmethod
    def _stop_batch(auth, appid, taskids):
        """停止任务"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/tasks/stop")
        http_method = "PUT"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=taskids,
                                       application_id=appid))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            # print (u"任务 {} 停止指令发送成功".format(','.join(taskids)))
            print (u"Command:stop {} sent successfully".format(','.join(taskids)))
        except Exception as e:
            raise

    @staticmethod
    def _delete_batch(auth, taskids):
        """删除任务"""
        auth_token = auth.auth_token
        access_authority_token = auth.access_authority_token
        address = auth.address
        security = auth.security
        url = "{}://{}{}".format(security, address, "/dispatch/platform/v1/tasks")
        http_method = "DELETE"
        headers = {"Authorization": auth_token, "X-Access-Authority": access_authority_token,
                   "Content-Type": "application/json"}
        request_body = json.dumps(dict(ids=taskids))
        try:
            status, response_data = http_request(url=url, http_method=http_method, body=request_body, headers=headers)
            data = json.loads(response_data)
            if status != 200:
                message = data.get("message", "Unknown Error")
                raise Exception(message)
            print (u"Delete task {} complete".format(','.join(taskids)))
        except Exception as e:
            raise




# -*- coding: utf-8 -*-
from .instance_ch import Instance as Ins_ch
from .instance_en import Instance as Ins_en
from .instance_json import Instance as Ins_json
from .base.instance import Instance as Ins_base


class InstanceDispatch(object):
    @staticmethod
    def list(auth, serviceid, format_json):
        """根据服务ID显示该应用下实例列表"""
        if format_json:
            Ins_json.list(auth, serviceid)
        else:
            if auth.language == "en":
                Ins_en.list(auth, serviceid)
            else:
                Ins_ch.list(auth, serviceid)

    @staticmethod
    def deploy_batch(auth, serviceid, microserviceid, instanceids):
        """部署实例"""
        Ins_base._deploy_batch(auth, serviceid, microserviceid, instanceids)

    @staticmethod
    def undeploy_batch(auth, serviceid, microserviceid, instanceids):
        """卸载实例"""
        Ins_base._undeploy_batch(auth, serviceid, microserviceid, instanceids)

    @staticmethod
    def delete_batch(auth, serviceid, microserviceid, instanceids):
        """批量删除实例"""
        Ins_base._delete_batch(auth, serviceid, microserviceid, instanceids)

    @staticmethod
    def add_batch(auth, serviceid, microserviceid, instanceids):
        """批量添加实例"""
        print("Developing...")

    @staticmethod
    def log(auth, instanceid):
        """实例日志"""
        Ins_base._log(auth, instanceid)

    @staticmethod
    def terminal(auth, instanceid):
        """实例终端"""
        Ins_base._terminal(auth, instanceid)


# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.file import FileDispatch as File


class UploadCommands(object):
    @args("--dest", dest="dest", default="/", help="")
    @args("--files", dest="files", required=True, nargs="+", help="")
    def file(self, auth, dest, files):
        """upload file"""
        File.upload(auth, dest, files)



# -*- coding: utf-8 -*-
import sys
reload(sys)
sys.setdefaultencoding('utf8')
from .client import Client


def main():
    try:
        Client().main(sys.argv[1:])
        # Client().main(["ls", "machine", "--clusterid", "c1"])
        # Client().main(["ls", "application"])
    except Exception as e:
        msg = e.message
        if not msg or msg == '':
            msg = "Unknown Error"
        print "ERROR: %s" % msg
        sys.exit(1)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.instance import Instance as Ins_base


lifecycle_conf_en = {"running": "Running",
                  "setting": "Setting",
                  "executing": "Executing"}
status_summary_conf_en = {"successfully_deployed": "Successfully deployed",
                       "failed_deployed": "Failed deployed",
                       "executing": "Executing",
                       "setting": "Setting"}
running_status_conf_en = {"green": "Normal",
                       "gray": "",
                       "yellow": "Warning",
                       "red": "Error"}


class Instance(object):
    @staticmethod
    def list(auth, serviceid):
        try:
            data = Ins_base._list(auth, serviceid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Name"
            header["status_summary_running"] = "Status"
            header["micro_service_id"] = "Microservice ID"
            header["service_id"] = "Service ID"
            header["application_id"] = "Application ID"
            header["cluster_id"] = "Cluster ID"
            header["node_id"] = "Node ID"
            header["node_name"] = "Node Name"
            header["data_path_instance"] = "Data Dir"
            header["create_time"] = "Create Time"
            instances_info = []
            microservices = data.get("micro_services", [])
            for microservice in microservices:
                for instance in microservice.get("instances", []):
                    instance["node_id"] = instance.get("node_id") or instance.get("running_node_id")
                    instance["node_name"] = instance.get("node_name") or instance.get("running_node_name")
                    status_summary_running = status_format(instance.get("lifecycle", ""), lifecycle_conf_en,
                                                           instance.get("status_summary", ""), status_summary_conf_en,
                                                           instance.get("running_status", ""), running_status_conf_en)
                    instance["status_summary_running"] = status_summary_running
                    instance = obj_replace_time_field(instance, ["create_time"])
                    instance_info = [instance.get(key, "") for key in header]
                    instances_info.append(instance_info)
            print_table(header.values(), instances_info, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-


import socket
import requests

origin_socket = socket.socket


def http_request(url, http_method, body=None, json=None, headers=None, proxy_info=None, timeout=60):
    proxies = None
    socket.socket = origin_socket
    if proxy_info:
        username = proxy_info.get("socks_proxy_username")
        password = proxy_info.get("socks_proxy_password")
        ip = proxy_info.get("proxy_ip")
        port = proxy_info.get("socks_proxy_port")

        proxy_info_str = "socks5://{}:{}@{}:{}".format(username, password, ip, port)
        proxies = {
            'http': proxy_info_str,
            'https': proxy_info_str
        }
    try:
        if http_method == "POST":
            res = requests.post(url, body, json, headers=headers, proxies=proxies, timeout=timeout)
        elif http_method == "GET":
            res = requests.get(url, headers=headers, proxies=proxies, timeout=timeout)
        elif http_method == "DELETE":
            res = requests.delete(url, data=body, headers=headers, proxies=proxies, timeout=timeout)
        elif http_method == "PATCH":
            res = requests.patch(url, body, headers=headers, proxies=proxies, timeout=timeout)
        elif http_method == "HEAD":
            res = requests.head(url, headers=headers, proxies=proxies, timeout=timeout)
        elif http_method == "PUT":
            res = requests.put(url, data=body, headers=headers, proxies=proxies, timeout=timeout)
        else:
            raise Exception("unsupported http method: {}".format(http_method))
        return res.status_code, res.text
    except Exception as e:
        raise Exception("http error, can't connect to http server url: {}, proxy_info: {}, error_info: {}".format(url, proxy_info, e.message))



# -*- coding: utf-8 -*-
import argparse
import collections
from .statiscommands import StatisCommands
from .lscommands import LsCommands
from .addcommands import AddCommands
from .deletecommands import DeleteCommands
from .deploycommands import DeployCommands
from .redeploycommands import RedeployCommands
from .undeploycommands import UndeployCommands
from .removecommands import RemoveCommands
from .startcommands import StartCommands
from .stopcommands import StopCommands
from .uploadcommands import UploadCommands
from .downloadcommands import DownloadCommands
from .utils.parse_utils import args, methods_of, fetch_func_args
from .v3_0.httpclient import HttpClient
from .v3_0.instance import InstanceDispatch as Instance


@args('-u', '--username', dest='username', help='use for auth')
@args('-p', '--password', dest='password', help='use for auth')
@args('-d', '--domain', dest='domain', type=str, choices=['huayun.com', 'bigdata.com'], help='use for auth')
@args('-a', '--address', dest='address', type=str, help='dataultra url')
@args('-s', '--security', dest='security', type=str, default='http', choices=['http', 'https'], help='protocol of dataultra address')
@args('-l', '--language', dest='language', type=str, default='ch', choices=['en', 'ch'], help='select language')
def login(username, password, domain, address, security, language):
    """login   [[-u|--username USERNAME]
                [-p|--password PASSWORD]
                [-d|--domain DOMAIN]
                [-a|--address ADDRESS]
                [-s|--security SECURITY(http,https)]| ]
                [-l|--language LANGUAGE(en,ch)]"""
    if username and password and domain and address:
        verify_conf = {"username": username,
                       "password": password,
                       "domain": domain,
                       "address": address,
                       "security": security,
                       "language": language}
        httpclient = HttpClient()
        httpclient.login(verify_conf)
    elif not (username or password or domain or address):
        httpclient = HttpClient()
        httpclient.login()
    else:
        raise Exception(u"Parameters error")


def logout():
    """logout"""
    httpclient = HttpClient()
    httpclient.logout()


@args('instanceid', help='instance id')
def _log(auth, instanceid):
    """show log of instance"""
    Instance.log(auth, instanceid)


@args('instanceid', help='instance id')
def _exec(auth, instanceid):
    """execute command on the terminal of instance"""
    Instance.terminal(auth, instanceid)


class Client(object):
    CATEGORIES_FUNC = {
        'login': login,
        'logout': logout,
        'log': _log,
        'exec': _exec
    }
    CATEGORIES = collections.OrderedDict()
    CATEGORIES["ls"] = (LsCommands, "list cluster|machine|application|service|task|microservice|instance info")
    CATEGORIES["statistics"] = (StatisCommands, "statistic cluster|machine|application|microservice")
    CATEGORIES["deploy"] = (DeployCommands, "deploy machine|service|instance|task")
    CATEGORIES["redeploy"] = (RedeployCommands, "redeploy machine|service|instance|task")
    CATEGORIES["undeploy"] = (UndeployCommands, "undeploy machine|service|instance|task")
    CATEGORIES["delete"] = (DeleteCommands, "delete cluster|machine|application|service|task|microservice|instance")
    CATEGORIES["add"] = (AddCommands, "add  machine|instance")
    CATEGORIES["remove"] = (RemoveCommands, "remove machine")
    CATEGORIES["start"] = (StartCommands, "start task")
    CATEGORIES["stop"] = (StopCommands, "stop task")
    CATEGORIES["upload"] = (UploadCommands, "upload file")
    CATEGORIES["download"] = (DownloadCommands, "download file")

    def get_base_parser(self):
        parser = argparse.ArgumentParser(prog="suctl")
        return parser

    def get_subcommand_parser(self):
        top_parser = self.get_base_parser()
        subparsers = top_parser.add_subparsers()

        for category_func in self.CATEGORIES_FUNC:
            func = self.CATEGORIES_FUNC[category_func]
            command_object = func
            category_func_parser = subparsers.add_parser(category_func, help=command_object.__doc__)

            action_kwargs = []
            for args, kwargs in getattr(command_object, 'args', []):
                category_func_parser.add_argument(*args, **kwargs)
            category_func_parser.set_defaults(action_fn=command_object)
            category_func_parser.set_defaults(action_kwargs=action_kwargs)

        for category in self.CATEGORIES:
            cls, help_mes = self.CATEGORIES[category]
            command_object = cls()

            category_parser = subparsers.add_parser(category, help=help_mes)
            category_parser.set_defaults(command_object=command_object)

            category_subparsers = category_parser.add_subparsers(dest='action')
            for (action, action_fn, action_help) in methods_of(command_object):
                parser = category_subparsers.add_parser(action, help=action_help)

                action_kwargs = []
                for args, kwargs in getattr(action_fn, 'args', []):
                    parser.add_argument(*args, **kwargs)

                parser.set_defaults(action_fn=action_fn)
                parser.set_defaults(action_kwargs=action_kwargs)
        return top_parser

    def main(self, argv):
        parser = self.get_subcommand_parser()
        args = parser.parse_args(argv)

        try:
            fn = getattr(args, "action_fn", None)
            if fn:
                fn_args = fetch_func_args(fn, args)
                if fn.func_name != "login" and fn.func_name != "logout":
                    authclient = HttpClient()
                    authclient.get_token()
                    fn(authclient, *fn_args)
                else:
                    fn(*fn_args)
            else:
                parser.print_help()
        except Exception as e:
            raise



# -*- encoding: UTF-8 -*-
from setuptools import setup, find_packages

"""
打包的用的setup必须引入，
"""

VERSION = '0.1.4'

setup(name='serviceultractl',
      version=VERSION,
      # package_data={'danmufm': ['template/*', ]},
      description="a tiny and smart cli player of dataultra based on Python",
      long_description='just enjoy',
      classifiers=[],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='python serviceultra su.cli terminal',
      author='su',
      author_email='du@huayun.com',
      url='http://demo.su.huayun.com/',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=True,
      install_requires=[
          'requests',
          'prettytable',
  	  'requests_toolbelt'
      ],
      entry_points={
          'console_scripts': [
              'suctl = serviceultractl.shell:main'
          ]
      },
      )



# -*- coding: utf-8 -*-
import collections
from ..utils.common_utils import print_table, obj_replace_time_field, status_format
from .base.microservice import MicroService as Ms_base


lifecycle_conf_en = {"running": "Running",
                     "setting": "Setting",
                     "executing": "Executing"}
status_summary_conf_en = {"successfully_deployed": "Successfully deployed",
                          "failed_deployed": "Failed deployed",
                          "executing": "Executing",
                          "setting": "Setting"}
running_status_conf_en = {"green": "Normal",
                          "gray": "",
                          "yellow": "Warning",
                          "red": "Error"}


class MicroService(object):
    @staticmethod
    def list(auth, serviceid):
        try:
            data = Ms_base._list(auth, serviceid)
            header = collections.OrderedDict()
            header["id"] = "ID"
            header["name"] = "Microservice Name"
            header["status_summary_running"] = "Status"
            header["service_id"] = "Service ID"
            header["application_id"] = "Application ID"
            header["cluster_id"] = "Cluster ID"
            header["prototype"] = "Service Type"
            header["instance_num"] = "Instance Num"
            header["version"] = "Version"
            header["cpu"] = "CPU(cor)"
            header["memory"] = "Memory(M)"
            header["create_time"] = "Create Time"
            microservices_info = []
            microservices = data.get("micro_services", [])
            for microservice in microservices:
                microservice = obj_replace_time_field(microservice, ["create_time"])
                status_summary_running = status_format(microservice.get("lifecycle", ""), lifecycle_conf_en,
                                                       microservice.get("status_summary", ""), status_summary_conf_en,
                                                       microservice.get("running_status", ""), running_status_conf_en)
                microservice["status_summary_running"] = status_summary_running
                microservice_info = [microservice.get(key, "") for key in header]
                microservices_info.append(microservice_info)
            print_table(header.values(), microservices_info, auth.encoding)
        except Exception as e:
            raise

    @staticmethod
    def statistics(auth, serviceid):
        try:
            data = Ms_base._statistics(auth, serviceid)
            header = collections.OrderedDict()
            header["cpu"] = "CPU(cor)"
            header["instance_num"] = "Instance Num"
            header["memory"] = "Memory(M)"
            header["micro_service_num"] = "Micro Service Num"
            statistics_infos = []
            statistics = data.get("statistics", {})
            statistics_info = [statistics.get(key, "") for key in header]
            statistics_infos.append(statistics_info)
            print_table(header.values(), statistics_infos, auth.encoding)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from .file_ch import File as File_ch
from .file_en import File as File_en
from .file_json import File as File_json
from .base.file import File as File_base


class FileDispatch(object):
    @staticmethod
    def list(auth, path, format_json):
        if format_json:
            File_json.list(auth, path)
        else:
            if auth.language == "en":
                File_en.list(auth, path)
            else:
                File_ch.list(auth, path)

    @staticmethod
    def upload(auth, dest, files):
        File_base._upload(auth, dest, files)

    @staticmethod
    def download(auth, path):
        File_base._download(auth, path)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from .base.task import Task as Task_base


class Task(object):
    @staticmethod
    def list(auth, appid):
        try:
            data = Task_base._list(auth, appid)
            tasks_info = []
            tasks = data.get("tasks", [])
            for task in tasks:
                task_info = dict(id=task.get("id", ""),
                                 name=task.get("name", ""),
                                 lifecycle=task.get("lifecycle", ""),
                                 running_status=task.get("running_status", ""),
                                 application_id=task.get("application_id", ""),
                                 cluster_id=task.get("cluster_id", ""),
                                 cluster_name=task.get("cluster_name", ""),
                                 version=task.get("version", ""),
                                 prototype=task.get("prototype", ""),
                                 resource_info=task.get("resource_info", ""),
                                 schedule=task.get("schedule", ""),
                                 create_time=task.get("create_time", ""),
                                 )
                task_info.get("resource_info", {}).pop("disk_allocated", "")
                task_info.get("resource_info", {}).pop("disk_total", "")
                tasks_info.append(task_info)
            print json.dumps(dict(tasks=tasks_info), indent=4, ensure_ascii=False)
        except Exception as e:
            raise



# -*- coding: utf-8 -*-
from .utils.parse_utils import args
from .v3_0.cluster import ClusterDispatch as Cluster
from .v3_0.commonnode import CommonNodeDispatch as CN
from .v3_0.application import ApplicationDispatch as Application
from .v3_0.service import ServiceDispatch as Service
from .v3_0.task import TaskDispatch as Task
from .v3_0.microservice import MicroServiceDispatch as MicroService
from .v3_0.instance import InstanceDispatch as Instance


class DeleteCommands(object):
    @args("--clusterid", dest="clusterid", required=True, help="")
    def cluster(self, auth, clusterid):
        """delete cluster"""
        Cluster.delete(auth, clusterid)

    @args("--machineid", dest="machineids", required=True, nargs="+", help="")
    def machine(self, auth, machineids):
        """delete machine"""
        CN.delete_batch(auth, machineids)

    @args("--appid", dest="appid", required=True, help="")
    def application(self, auth, appid):
        """delete application"""
        Application.delete(auth, appid)

    @args("--serviceid", dest="serviceids", required=True, nargs="+", help="")
    @args("--appid", dest="appid", required=True, help="")
    def service(self, auth, serviceids, appid):
        """delete service"""
        Service.delete_batch(auth, appid, serviceids)

    # @args("--taskid", dest="taskids", required=True, nargs="+", help=u"任务ID")
    # @args("--appid", dest="appid", required=True, help=u"所属应用ID")
    # def task(self, auth, taskids, appid):
    #     """删除任务"""
    #     Task.delete_batch(auth, appid, taskids)
    @args("--taskid", dest="taskids", required=True, nargs="+", help="")
    def task(self, auth, taskids):
        """delete task"""
        Task.delete_batch(auth, taskids)

    @args("--microserviceid", dest="microserviceids", required=True, nargs="+", help="")
    @args("--serviceid", dest="serviceid", required=True, help="")
    def microservice(self, auth, microserviceids, serviceid):
        """delete micro service"""
        MicroService.delete_batch(auth, serviceid, microserviceids)

    @args("--instanceid", dest="instanceids", required=True, nargs="+", help="")
    @args("--microserviceid", dest="microserviceid", required=True, help="")
    @args("--serviceid", dest="serviceid", required=True, help="")
    def instance(self, auth, instanceids, microserviceid, serviceid):
        """delete instance"""
        Instance.delete_batch(auth, serviceid, microserviceid, instanceids)




# -*- coding: utf-8 -*-
from .task_ch import Task as Task_ch
from .task_en import Task as Task_en
from .task_json import Task as Task_json
from .base.task import Task as Task_base


class TaskDispatch(object):
    @staticmethod
    def list(auth, appid, format_json):
        """根据应用ID显示该应用下任务列表"""
        if format_json:
            Task_json.list(auth, appid)
        else:
            if auth.language == "en":
                Task_en.list(auth, appid)
            else:
                Task_ch.list(auth, appid)

    @staticmethod
    def deploy_batch(auth, appid, taskids):
        """任务上线"""
        Task_base._deploy_batch(auth, appid, taskids)

    @staticmethod
    def undeploy_batch(auth, appid, taskids):
        """任务下线"""
        Task_base._undeploy_batch(auth, appid, taskids)

    @staticmethod
    def start_batch(auth, appid, taskids):
        """启动任务"""
        Task_base._start_batch(auth, appid, taskids)

    @staticmethod
    def stop_batch(auth, appid, taskids):
        """停止任务"""
        Task_base._stop_batch(auth, appid, taskids)

    @staticmethod
    def delete_batch(auth, taskids):
        """删除任务"""
        Task_base._delete_batch(auth, taskids)

